@@ -1,21 +1,24 @@
-#!/usr/bin/env python -u
+#!/usr/bin/env python
 
 """Hang Analyzer
 
-A prototype hang analyzer for MCI integration to help investigate test timeouts
+A prototype hang analyzer for Evergreen integration to help investigate test timeouts
 
 1. Script supports taking dumps, and/or dumping a summary of useful information about a process
-2. Script will iterate through a list of interesting processes (mongo, mongod, and mongos),
-    and run the tools from step 1.
+2. Script will iterate through a list of interesting processes,
+    and run the tools from step 1. The list of processes can be provided as an option.
+3. Java processes will be dumped using jstack, if available.
 
 Supports Linux, MacOS X, Solaris, and Windows.
 """
 
 import StringIO
 import csv
+import glob
 import itertools
 import os
 import platform
+import re
 import signal
 import subprocess
 import sys
@@ -110,7 +113,7 @@ class WindowsDumper(object):
                 return os.path.join(pathToTest, "cdb.exe")
         return None
 
-    def dump_info(self, pid, process_name, stream):
+    def dump_info(self, pid, process_name, stream, take_dump = False):
         """Dump useful information to the console"""
         dbg = self.__find_debugger()
 
@@ -122,12 +125,13 @@ class WindowsDumper(object):
 
         cmds = [
             ".symfix",  # Fixup symbol path
+            ".symopt +0x10", # Enable line loading (off by default in CDB, on by default in WinDBG)
             ".reload",  # Reload symbols
             "!peb",     # Dump current exe, & environment variables
             "lm",       # Dump loaded modules
-            "~* k 100", # Dump All Threads
-            ".dump /ma /u dump_" + process_name + ".mdmp",
-                        # Dump to file, dump_<process name>_<time stamp>_<pid in hex>.mdmp
+            "~* kp 100", # Dump All Threads with function arguments
+            ".dump /ma /u dump_" + process_name + "." + str(pid) + "." + self.get_dump_ext() if take_dump else "",
+                        # Dump to file, dump_<process name>_<time stamp>_<pid in hex>.<pid>.mdmp
             ".detach",  # Detach
             "q"         # Quit
             ]
@@ -136,6 +140,9 @@ class WindowsDumper(object):
 
         stream.write("INFO: Done analyzing process\n")
 
+    def get_dump_ext(self):
+        return "mdmp"
+
     def dump_core(self, pid, output_file):
         """Take a dump of pid to specified file"""
         dbg = self.__find_debugger()
@@ -180,7 +187,7 @@ class LLDBDumper(object):
         """Finds the installed debugger"""
         return find_program('lldb', ['/usr/bin'])
 
-    def dump_info(self, pid, process_name, stream):
+    def dump_info(self, pid, process_name, stream, take_dump = False):
         dbg = self.__find_debugger()
 
         if dbg is None:
@@ -211,6 +218,7 @@ class LLDBDumper(object):
             "attach -p %d" % pid,
             "target modules list",
             "thread backtrace all",
+            "process save-core dump_" + process_name + "." + str(pid) + "." + self.get_dump_ext() if take_dump else "",
             "settings set interpreter.prompt-on-quit false",
             "quit",
             ]
@@ -229,6 +237,9 @@ class LLDBDumper(object):
 
         stream.write("INFO: Done analyzing process\n")
 
+    def get_dump_ext(self):
+        return "core"
+
     def dump_core(self, pid, output_file):
         """Take a dump of pid to specified file"""
         sys.stderr.write("ERROR: lldb does not support dumps, stupid debugger\n")
@@ -261,9 +272,9 @@ class GDBDumper(object):
 
     def __find_debugger(self):
         """Finds the installed debugger"""
-        return find_program('gdb', ['/opt/mongodbtoolchain/bin', '/usr/bin'])
+        return find_program('gdb', ['/opt/mongodbtoolchain/gdb/bin', '/usr/bin'])
 
-    def dump_info(self, pid, process_name, stream):
+    def dump_info(self, pid, process_name, stream, take_dump = False):
         dbg = self.__find_debugger()
 
         if dbg is None:
@@ -275,8 +286,11 @@ class GDBDumper(object):
         call([dbg, "--version"])
 
         cmds = [
+            "set pagination off",
             "attach %d" % pid,
+            "info sharedlibrary",
             "thread apply all bt",
+            "gcore dump_" + process_name + "." + str(pid) + "." + self.get_dump_ext() if take_dump else "",
             "set confirm off",
             "quit",
             ]
@@ -285,6 +299,9 @@ class GDBDumper(object):
 
         stream.write("INFO: Done analyzing process\n")
 
+    def get_dump_ext(self):
+        return "core"
+
     def _find_gcore(self):
         """Finds the installed gcore"""
         dbg = "/usr/bin/gcore"
@@ -356,45 +373,85 @@ class SolarisProcessList(object):
 
         return p
 
+# jstack is a JDK utility
+class JstackDumper(object):
+
+    def __find_debugger(self):
+        """Finds the installed jstack debugger"""
+        return find_program('jstack', ['/usr/bin'])
+
+    def dump_info(self, pid, process_name, stream, take_dump = False):
+        """Dump java thread stack traces to the console"""
+        jstack = self.__find_debugger()
+
+        if jstack is None:
+            stream.write("WARNING: Debugger jstack not found, skipping dumping of %d\n" % (pid))
+            return
+
+        stream.write("INFO: Debugger %s, analyzing %d\n" % (jstack, pid))
+
+        call([jstack, "-l", str(pid)])
+
+        stream.write("INFO: Done analyzing process\n")
+
+    def dump_core(self, pid, output_file):
+        """Take a dump of pid to specified file"""
+        sys.stderr.write("ERROR: jstack does not support dumps\n")
+
+
+# jstack is a JDK utility
+class JstackWindowsDumper(object):
+
+    def dump_info(self, pid, process_name, stream):
+        """Dump java thread stack traces to the console"""
+
+        stream.write("WARNING: Debugger jstack not supported, skipping dumping of %d\n" % (pid))
+
 def get_hang_analyzers():
     dbg = None
+    jstack = None
     ps = None
     if sys.platform.startswith("linux"):
         dbg = GDBDumper()
+        jstack = JstackDumper()
         ps = LinuxProcessList()
     elif sys.platform.startswith("sunos"):
         dbg = GDBDumper()
+        jstack = JstackDumper()
         ps = SolarisProcessList()
     elif os.name == 'nt' or (os.name == "posix" and sys.platform == "cygwin"):
         dbg = WindowsDumper()
+        jstack = JstackWindowsDumper()
         ps = WindowsProcessList()
     elif sys.platform == "darwin":
         dbg = LLDBDumper()
+        jstack = JstackDumper()
         ps = DarwinProcessList()
-    return [ps, dbg]
+    return [ps, dbg, jstack]
 
+def check_dump_quota(quota, ext):
+    """Check if sum of the files with ext is within the specified quota in megabytes"""
 
-interesting_processes = ["mongo", "mongod", "mongos", "_test", "dbtest", "python"]
+    files = glob.glob("*." + ext)
 
-def is_interesting_process(p):
-    for ip in interesting_processes:
-        if p.find(ip) != -1:
-            return True
-    return False
+    size_sum = 0
+    for file_name in files:
+        size_sum += os.path.getsize(file_name)
 
+    return (size_sum <= quota)
 
-def signal_process(pid):
-    """Signal python process with SIGUSR1, N/A on Windows"""
+def signal_process(pid, signalnum):
+    """Signal process with signal, N/A on Windows"""
     try:
-        os.kill(pid, signal.SIGUSR1)
+        os.kill(pid, signalnum)
 
-        print "Waiting for python process to report"
+        print "Waiting for process to report"
         time.sleep(5)
     except OSError,e:
-        print "Hit OS error trying to signal python process: " + str(e)
+        print "Hit OS error trying to signal process: " + str(e)
 
     except AttributeError:
-        print "Cannot send signal to python on Windows"
+        print "Cannot send signal to a process on Windows"
 
 
 def timeout_protector():
@@ -432,12 +489,26 @@ def main():
     except AttributeError:
         print "Cannot determine Unix Current Login, not supported on Windows"
 
+    interesting_processes = ["mongo", "mongod", "mongos", "_test", "dbtest", "python", "java"]
+    go_processes = []
+
     parser = OptionParser(description=__doc__)
+    parser.add_option('-p', '--process_names', dest='process_names', help='List of process names to analyze')
+    parser.add_option('-g', '--go_process_names', dest='go_process_names', help='List of go process names to analyze')
+    parser.add_option('-s', '--max_core_dumps_size', dest='max_core_dumps_size', default=10000, help='Maximum total size of core dumps to keep in megabytes')
+
     (options, args) = parser.parse_args()
 
-    [ps, dbg] = get_hang_analyzers()
+    if options.process_names is not None:
+        interesting_processes = options.process_names.split(',')
 
-    if( ps == None or dbg == None):
+    if options.go_process_names is not None:
+        go_processes = options.go_process_names.split(',')
+        interesting_processes += go_processes
+
+    [ps, dbg, jstack] = get_hang_analyzers()
+
+    if( ps == None or (dbg == None and jstack == None)):
         sys.stderr.write("hang_analyzer.py: Unsupported platform: %s\n" % (sys.platform))
         exit(1)
 
@@ -447,25 +518,42 @@ def main():
 
     processes_orig = ps.dump_processes()
 
-    # Pick unit tests which include the phrase "_test"
+    # Find all running interesting processes by doing a substring match.
     processes = [a for a in processes_orig
-                    if is_interesting_process(a[1]) and a[0] != os.getpid()]
+                    if any([a[1].find(ip) >= 0 for ip in interesting_processes]) and a[0] != os.getpid()]
 
     sys.stdout.write("Found %d interesting processes\n" % len(processes))
 
+    max_dump_size_bytes = int(options.max_core_dumps_size) * 1024 * 1024
+
     if( len(processes) == 0):
         for process in processes_orig:
             sys.stdout.write("Ignoring process %d of %s\n" % (process[0], process[1]))
     else:
-        # Dump all other processes first since signaling the python script interrupts it
-        for process in [a for a in processes if not a[1].startswith("python")]:
+        # Dump all other processes including go programs, except python & java.
+        for process in [a for a in processes if not re.match("^(java|python)", a[1])]:
             sys.stdout.write("Dumping process %d of %s\n" % (process[0], process[1]))
-            dbg.dump_info(process[0], process[1], sys.stdout)
+            dbg.dump_info(process[0], process[1], sys.stdout, check_dump_quota(max_dump_size_bytes, dbg.get_dump_ext()))
 
+        # Dump java processes using jstack.
+        for process in [a for a in processes if a[1].startswith("java")]:
+            sys.stdout.write("Dumping process %d of %s\n" % (process[0], process[1]))
+            jstack.dump_info(process[0], process[1], sys.stdout)
+
+        # Signal go processes to ensure they print out stack traces, and die on POSIX OSes.
+        # On Windows, this will simply kill the process since python emulates SIGABRT as
+        # TerminateProcess.
+        # Note: The stacktrace output may be captured elsewhere (i.e. resmoke).
+        for process in [a for a in processes if a[1] in go_processes]:
+            sys.stdout.write("Sending signal SIGABRT to go process %d of %s\n" % (process[0], process[1]))
+            signal_process(process[0], signal.SIGABRT)
+
+        # Dump python processes after signalling them.
         for process in [a for a in processes if a[1].startswith("python")]:
-            signal_process(process[0])
+            sys.stdout.write("Sending signal SIGUSR1 to python process %d of %s\n" % (process[0], process[1]))
+            signal_process(process[0], signal.SIGUSR1)
 
-            dbg.dump_info(process[0], process[1], sys.stdout)
+            dbg.dump_info(process[0], process[1], sys.stdout, check_dump_quota(max_dump_size_bytes, dbg.get_dump_ext()))
 
     # Suspend the timer so we can exit cleanly
     timer.cancel()
