@@ -2,13 +2,46 @@
  * The ParallelTester class is used to test more than one test concurrently
  */
 if (typeof _threadInject != "undefined") {
+    // With --enableJavaScriptProtection functions are presented as Code objects.
+    // This function evals all the Code objects then calls the provided start function.
+    // arguments: [startFunction, startFunction args...]
+    function _threadStartWrapper() {
+        // Recursively evals all the Code objects present in arguments
+        // NOTE: This is a naive implementation that cannot handle cyclic objects.
+        function evalCodeArgs(arg) {
+            if (arg instanceof Code) {
+                return eval("(" + arg.code + ")");
+            } else if (arg !== null && isObject(arg)) {
+                var newArg = arg instanceof Array ? [] : {};
+                for (var prop in arg) {
+                    if (arg.hasOwnProperty(prop)) {
+                        newArg[prop] = evalCodeArgs(arg[prop]);
+                    }
+                }
+                return newArg;
+            }
+            return arg;
+        }
+        var realStartFn;
+        var newArgs = [];
+        for (var i = 0, l = arguments.length; i < l; i++) {
+            newArgs.push(evalCodeArgs(arguments[i]));
+        }
+        realStartFn = newArgs.shift();
+        return realStartFn.apply(this, newArgs);
+    }
+
     Thread = function() {
-        this.init.apply(this, arguments);
+        var args = Array.prototype.slice.call(arguments);
+        args.unshift(_threadStartWrapper);
+        this.init.apply(this, args);
     };
     _threadInject(Thread.prototype);
 
     ScopedThread = function() {
-        this.init.apply(this, arguments);
+        var args = Array.prototype.slice.call(arguments);
+        args.unshift(_threadStartWrapper);
+        this.init.apply(this, args);
     };
     ScopedThread.prototype = new Thread(function() {});
     _scopedThreadInject(ScopedThread.prototype);
@@ -63,7 +96,7 @@ if (typeof _threadInject != "undefined") {
     };
 
     EventGenerator.dispatch = function() {
-        var args = argumentsToArray(arguments);
+        var args = Array.from(arguments);
         var me = args.shift();
         var collectionName = args.shift();
         var host = args.shift();
@@ -118,22 +151,14 @@ if (typeof _threadInject != "undefined") {
             "extent.js",
             "indexb.js",
 
-            // sets a failpoint that causes the server to ignore long keys,
-            // which makes index_bigkeys.js fail
+            // Tests that set a parameter that causes the server to ignore
+            // long index keys.
             "index_bigkeys_nofail.js",
-
-            // tests turn on profiling
-            "profile1.js",
-            "profile3.js",
-            "profile4.js",
-            "profile5.js",
-            "geo_s2cursorlimitskip.js",
+            "index_bigkeys_validation.js",
 
             "mr_drop.js",
             "mr3.js",
             "indexh.js",
-            "apitest_db.js",
-            "evalb.js",
             "evald.js",
             "evalf.js",
             "killop.js",
@@ -141,7 +166,6 @@ if (typeof _threadInject != "undefined") {
             "notablescan.js",
             "drop2.js",
             "dropdb_race.js",
-            "fsync2.js",  // May be placed in serialTestsArr once SERVER-4243 is fixed.
             "bench_test1.js",
             "padding.js",
             "queryoptimizera.js",
@@ -171,11 +195,31 @@ if (typeof _threadInject != "undefined") {
             parallelFilesDir + "/fsync.js",
             parallelFilesDir + "/auth1.js",
 
-            // These tests expect the profiler to be on or off at specific points
-            // during the test run.
-            parallelFilesDir + "/cursor6.js",
+            // These tests expect the profiler to be on or off at specific points. They should not
+            // be run in parallel with tests that peform fsyncLock. User operations skip writing to
+            // the system.profile collection while the server is fsyncLocked.
+            //
+            // The profiler tests can be run in parallel with each other as they use test-specific
+            // databases.
+            parallelFilesDir + "/apitest_db.js",
+            parallelFilesDir + "/evalb.js",
+            parallelFilesDir + "/geo_s2cursorlimitskip.js",
+            parallelFilesDir + "/profile1.js",
             parallelFilesDir + "/profile2.js",
-            parallelFilesDir + "/updatee.js"
+            parallelFilesDir + "/profile3.js",
+            parallelFilesDir + "/profile_agg.js",
+            parallelFilesDir + "/profile_count.js",
+            parallelFilesDir + "/profile_delete.js",
+            parallelFilesDir + "/profile_distinct.js",
+            parallelFilesDir + "/profile_find.js",
+            parallelFilesDir + "/profile_findandmodify.js",
+            parallelFilesDir + "/profile_geonear.js",
+            parallelFilesDir + "/profile_getmore.js",
+            parallelFilesDir + "/profile_group.js",
+            parallelFilesDir + "/profile_insert.js",
+            parallelFilesDir + "/profile_mapreduce.js",
+            parallelFilesDir + "/profile_no_such_db.js",
+            parallelFilesDir + "/profile_update.js"
         ];
         var serialTests = makeKeys(serialTestsArr);
 
@@ -211,7 +255,7 @@ if (typeof _threadInject != "undefined") {
     // runs a set of test files
     // first argument is an identifier for this tester, remaining arguments are file names
     ParallelTester.fileTester = function() {
-        var args = argumentsToArray(arguments);
+        var args = Array.from(arguments);
         var suite = args.shift();
         args.forEach(function(x) {
             print("         S" + suite + " Test : " + x + " ...");
