@@ -34,7 +34,6 @@
 #include <map>
 #include <memory>
 #include <string>
-#include <unordered_set>
 #include <vector>
 
 #include "mongo/base/string_data.h"
@@ -46,6 +45,7 @@
 #include "mongo/dbtests/mock/mock_conn_registry.h"
 #include "mongo/dbtests/mock/mock_replica_set.h"
 #include "mongo/rpc/metadata/server_selection_metadata.h"
+#include "mongo/stdx/unordered_set.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
 
@@ -77,9 +77,6 @@ protected:
     void setUp() {
         ReplicaSetMonitor::cleanup();
 
-        // Set the number of consecutive failed checks to 2 so the test doesn't run too long
-        ReplicaSetMonitor::maxConsecutiveFailedChecks = 2;
-
         _replSet.reset(new MockReplicaSet("test", 2));
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
     }
@@ -102,14 +99,14 @@ private:
 void assertOneOfNodesSelected(MockReplicaSet* replSet,
                               ReadPreference rp,
                               const std::vector<std::string> hostNames) {
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
     ReplicaSetMonitor::get(replSet->getSetName())->startOrContinueRefresh().refreshAll();
     bool secondaryOk = (rp != ReadPreference::PrimaryOnly);
     auto tagSet = secondaryOk ? TagSet() : TagSet::primaryOnly();
     // We need the command to be a "SecOk command"
     auto res = replConn.runCommandWithMetadata(
         "foo", "dbStats", makeMetadata(rp, tagSet, secondaryOk), BSON("dbStats" << 1));
-    std::unordered_set<HostAndPort> hostSet;
+    stdx::unordered_set<HostAndPort> hostSet;
     for (const auto& hostName : hostNames) {
         hostSet.emplace(hostName);
     }
@@ -122,7 +119,7 @@ void assertNodeSelected(MockReplicaSet* replSet, ReadPreference rp, StringData h
 
 TEST_F(BasicRS, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
@@ -139,7 +136,7 @@ TEST_F(BasicRS, CommandPrimary) {
 
 TEST_F(BasicRS, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
@@ -157,7 +154,7 @@ TEST_F(BasicRS, CommandSecondaryOnly) {
 
 TEST_F(BasicRS, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     // Need up-to-date view, since either host is valid if view is stale.
     ReplicaSetMonitor::get(replSet->getSetName())->startOrContinueRefresh().refreshAll();
@@ -177,7 +174,7 @@ TEST_F(BasicRS, CommandPrimaryPreferred) {
 
 TEST_F(BasicRS, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     // Need up-to-date view, since either host is valid if view is stale.
     ReplicaSetMonitor::get(replSet->getSetName())->startOrContinueRefresh().refreshAll();
@@ -204,9 +201,6 @@ protected:
     void setUp() {
         ReplicaSetMonitor::cleanup();
 
-        // Set the number of consecutive failed checks to 2 so the test doesn't run too long
-        ReplicaSetMonitor::maxConsecutiveFailedChecks = 2;
-
         _replSet.reset(new MockReplicaSet("test", 2));
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
 
@@ -238,7 +232,7 @@ void assertRunCommandWithReadPrefThrows(MockReplicaSet* replSet, ReadPreference
     bool secondaryOk = !isPrimaryOnly;
     TagSet ts = isPrimaryOnly ? TagSet::primaryOnly() : TagSet();
 
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
     ASSERT_THROWS(replConn.runCommandWithMetadata(
                       "foo", "whoami", makeMetadata(rp, ts, secondaryOk), BSON("dbStats" << 1)),
                   AssertionException);
@@ -246,7 +240,7 @@ void assertRunCommandWithReadPrefThrows(MockReplicaSet* replSet, ReadPreference
 
 TEST_F(AllNodesDown, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
@@ -259,7 +253,7 @@ TEST_F(AllNodesDown, CommandPrimary) {
 
 TEST_F(AllNodesDown, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
@@ -272,7 +266,7 @@ TEST_F(AllNodesDown, CommandSecondaryOnly) {
 
 TEST_F(AllNodesDown, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
@@ -285,7 +279,7 @@ TEST_F(AllNodesDown, CommandPrimaryPreferred) {
 
 TEST_F(AllNodesDown, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
@@ -298,7 +292,7 @@ TEST_F(AllNodesDown, CommandSecondaryPreferred) {
 
 TEST_F(AllNodesDown, QueryNearest) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::Nearest, BSONArray());
@@ -317,9 +311,6 @@ protected:
     void setUp() {
         ReplicaSetMonitor::cleanup();
 
-        // Set the number of consecutive failed checks to 2 so the test doesn't run too long
-        ReplicaSetMonitor::maxConsecutiveFailedChecks = 2;
-
         _replSet.reset(new MockReplicaSet("test", 2));
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
         _replSet->kill(_replSet->getPrimary());
@@ -342,7 +333,7 @@ private:
 
 TEST_F(PrimaryDown, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
@@ -355,7 +346,7 @@ TEST_F(PrimaryDown, CommandPrimary) {
 
 TEST_F(PrimaryDown, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
@@ -373,7 +364,7 @@ TEST_F(PrimaryDown, CommandSecondaryOnly) {
 
 TEST_F(PrimaryDown, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
@@ -391,7 +382,7 @@ TEST_F(PrimaryDown, CommandPrimaryPreferred) {
 
 TEST_F(PrimaryDown, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
@@ -409,7 +400,7 @@ TEST_F(PrimaryDown, CommandSecondaryPreferred) {
 
 TEST_F(PrimaryDown, Nearest) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::Nearest, BSONArray());
@@ -426,9 +417,6 @@ protected:
     void setUp() {
         ReplicaSetMonitor::cleanup();
 
-        // Set the number of consecutive failed checks to 2 so the test doesn't run too long
-        ReplicaSetMonitor::maxConsecutiveFailedChecks = 2;
-
         _replSet.reset(new MockReplicaSet("test", 2));
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
 
@@ -452,7 +440,7 @@ private:
 
 TEST_F(SecondaryDown, QueryPrimary) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryOnly, BSONArray());
@@ -469,7 +457,7 @@ TEST_F(SecondaryDown, CommandPrimary) {
 
 TEST_F(SecondaryDown, QuerySecondaryOnly) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryOnly, BSONArray());
@@ -482,7 +470,7 @@ TEST_F(SecondaryDown, CommandSecondaryOnly) {
 
 TEST_F(SecondaryDown, QueryPrimaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::PrimaryPreferred, BSONArray());
@@ -499,7 +487,7 @@ TEST_F(SecondaryDown, CommandPrimaryPreferred) {
 
 TEST_F(SecondaryDown, QuerySecondaryPreferred) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::SecondaryPreferred, BSONArray());
@@ -516,7 +504,7 @@ TEST_F(SecondaryDown, CommandSecondaryPreferred) {
 
 TEST_F(SecondaryDown, QueryNearest) {
     MockReplicaSet* replSet = getReplSet();
-    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts());
+    DBClientReplicaSet replConn(replSet->getSetName(), replSet->getHosts(), StringData());
 
     Query query;
     query.readPref(mongo::ReadPreference::Nearest, BSONArray());
@@ -545,9 +533,6 @@ protected:
         // tests depend on controlling when the RSMs are updated.
         ReplicaSetMonitor::cleanup();
 
-        // Set the number of consecutive failed checks to 2 so the test doesn't run too long
-        ReplicaSetMonitor::maxConsecutiveFailedChecks = 2;
-
         _replSet.reset(new MockReplicaSet("test", 5));
         _originalConnectionHook = ConnectionString::getConnectionHook();
         ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
@@ -662,7 +647,7 @@ TEST_F(TaggedFiveMemberRS, ConnShouldPinIfSameSettings) {
     vector<HostAndPort> seedList;
     seedList.push_back(HostAndPort(replSet->getPrimary()));
 
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList);
+    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
 
     string dest;
     {
@@ -690,7 +675,7 @@ TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfHostMarkedAsFailed) {
     vector<HostAndPort> seedList;
     seedList.push_back(HostAndPort(replSet->getPrimary()));
 
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList);
+    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
 
     string dest;
     {
@@ -724,7 +709,7 @@ TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfDiffMode) {
     vector<HostAndPort> seedList;
     seedList.push_back(HostAndPort(replSet->getPrimary()));
 
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList);
+    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
 
     // Need up-to-date view to ensure there are multiple valid choices.
     ReplicaSetMonitor::get(replSet->getSetName())->startOrContinueRefresh().refreshAll();
@@ -756,7 +741,7 @@ TEST_F(TaggedFiveMemberRS, ConnShouldNotPinIfDiffTag) {
     vector<HostAndPort> seedList;
     seedList.push_back(HostAndPort(replSet->getPrimary()));
 
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList);
+    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
 
     // Need up-to-date view to ensure there are multiple valid choices.
     ReplicaSetMonitor::get(replSet->getSetName())->startOrContinueRefresh().refreshAll();
@@ -792,7 +777,7 @@ TEST_F(TaggedFiveMemberRS, SlaveConnReturnsSecConn) {
     vector<HostAndPort> seedList;
     seedList.push_back(HostAndPort(replSet->getPrimary()));
 
-    DBClientReplicaSet replConn(replSet->getSetName(), seedList);
+    DBClientReplicaSet replConn(replSet->getSetName(), seedList, StringData());
 
     // Need up-to-date view since slaveConn() uses SecondaryPreferred, and this test assumes it
     // knows about at least one secondary.
