@@ -34,15 +34,18 @@
 #include "mongo/client/dbclientcursor.h"
 
 #include "mongo/client/connpool.h"
+#include "mongo/db/client.h"
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/rpc/factory.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata.h"
+#include "mongo/rpc/object_check.h"
 #include "mongo/rpc/request_builder_interface.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/debug_util.h"
+#include "mongo/util/destructor_guard.h"
 #include "mongo/util/exit.h"
 #include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
@@ -78,7 +81,10 @@ Message assembleCommandRequest(DBClientWithCommands* cli,
     BSONObjBuilder metadataBob;
     metadataBob.appendElements(upconvertedMetadata);
     if (cli->getRequestMetadataWriter()) {
-        uassertStatusOK(cli->getRequestMetadataWriter()(&metadataBob, cli->getServerAddress()));
+        uassertStatusOK(
+            cli->getRequestMetadataWriter()((haveClient() ? cc().getOperationContext() : nullptr),
+                                            &metadataBob,
+                                            cli->getServerAddress()));
     }
 
     requestBuilder->setDatabase(database);
@@ -179,16 +185,6 @@ bool DBClientCursor::initLazyFinish(bool& retry) {
     return !retry;
 }
 
-bool DBClientCursor::initCommand() {
-    BSONObj res;
-
-    bool ok = _client->runCommand(nsGetDB(ns), query, res, opts);
-    replyToQuery(0, batch.m, res);
-    dataReceived();
-
-    return ok;
-}
-
 void DBClientCursor::requestMore() {
     verify(cursorId && batch.pos == batch.nReturned);
 
@@ -269,6 +265,7 @@ void DBClientCursor::commandDataReceived() {
 
     QueryResult::View qr = batch.m.singleData().view2ptr();
     batch.data = qr.data();
+    batch.remainingBytes = qr.dataLen();
 }
 
 void DBClientCursor::dataReceived(bool& retry, string& host) {
@@ -307,6 +304,7 @@ void DBClientCursor::dataReceived(bool& retry, string& host) {
     batch.nReturned = qr.getNReturned();
     batch.pos = 0;
     batch.data = qr.data();
+    batch.remainingBytes = qr.dataLen();
 
     _client->checkResponse(batch.data, batch.nReturned, &retry, &host);  // watches for "not master"
 
@@ -324,8 +322,6 @@ void DBClientCursor::dataReceived(bool& retry, string& host) {
 
 /** If true, safe to call next().  Requests more from server if necessary. */
 bool DBClientCursor::more() {
-    _assertIfNull();
-
     if (!_putBack.empty())
         return true;
 
@@ -343,7 +339,6 @@ bool DBClientCursor::more() {
 }
 
 BSONObj DBClientCursor::next() {
-    DEV _assertIfNull();
     if (!_putBack.empty()) {
         BSONObj ret = _putBack.top();
         _putBack.pop();
@@ -352,9 +347,20 @@ BSONObj DBClientCursor::next() {
 
     uassert(13422, "DBClientCursor next() called but more() is false", batch.pos < batch.nReturned);
 
-    batch.pos++;
+    if (serverGlobalParams.objcheck) {
+        auto status = validateBSON(batch.data, batch.remainingBytes, _enabledBSONVersion);
+        uassert(ErrorCodes::InvalidBSON,
+                str::stream() << "Got invalid BSON from external server while reading from cursor"
+                              << causedBy(status),
+                status.isOK());
+    }
+
     BSONObj o(batch.data);
+
+    batch.pos++;
     batch.data += o.objsize();
+    batch.remainingBytes -= o.objsize();
+
     /* todo would be good to make data null at end of batch for safety */
     return o;
 }
@@ -432,8 +438,7 @@ void DBClientCursor::attach(AScopedConnection* conn) {
     verify(conn);
     verify(conn->get());
 
-    if (conn->get()->type() == ConnectionString::SET ||
-        conn->get()->type() == ConnectionString::SYNC) {
+    if (conn->get()->type() == ConnectionString::SET) {
         if (_lazyHost.size() > 0)
             _scopedHost = _lazyHost;
         else if (_client)
@@ -507,7 +512,8 @@ DBClientCursor::DBClientCursor(DBClientBase* client,
       resultFlags(0),
       cursorId(cursorId),
       _ownCursor(true),
-      wasError(false) {}
+      wasError(false),
+      _enabledBSONVersion(Validator<BSONObj>::enabledBSONVersion()) {}
 
 DBClientCursor::~DBClientCursor() {
     kill();
