@@ -40,7 +40,8 @@
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/server_parameters.h"
-#include "mongo/s/catalog/catalog_manager.h"
+#include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/grid.h"
 #include "mongo/stdx/mutex.h"
 #include "mongo/util/background.h"
@@ -92,10 +93,10 @@ public:
 StatusWith<OID> getCurrentCacheGeneration(OperationContext* txn) {
     try {
         BSONObjBuilder result;
-        const bool ok = grid.catalogManager(txn)->runUserManagementReadCommand(
+        const bool ok = grid.catalogClient(txn)->runUserManagementReadCommand(
             txn, "admin", BSON("_getUserCacheGeneration" << 1), &result);
         if (!ok) {
-            return Command::getStatusFromCommandResult(result.obj());
+            return getStatusFromCommandResult(result.obj());
         }
         return result.obj()["cacheGeneration"].OID();
     } catch (const DBException& e) {
@@ -129,7 +130,8 @@ void UserCacheInvalidator::initialize(OperationContext* txn) {
                      "running an outdated version of mongod on the config servers";
     } else {
         warning() << "An error occurred while fetching initial user cache generation from "
-                     "config servers: " << currentGeneration.getStatus();
+                     "config servers: "
+                  << currentGeneration.getStatus();
     }
     _previousCacheGeneration = OID();
 }
@@ -140,11 +142,12 @@ void UserCacheInvalidator::run() {
 
     while (true) {
         stdx::unique_lock<stdx::mutex> lock(invalidationIntervalMutex);
-        Date_t sleepUntil = lastInvalidationTime + Seconds(userCacheInvalidationIntervalSecs);
+        Date_t sleepUntil =
+            lastInvalidationTime + Seconds(userCacheInvalidationIntervalSecs.load());
         Date_t now = Date_t::now();
         while (now < sleepUntil) {
-            invalidationIntervalChangedCondition.wait_for(lock, sleepUntil - now);
-            sleepUntil = lastInvalidationTime + Seconds(userCacheInvalidationIntervalSecs);
+            invalidationIntervalChangedCondition.wait_until(lock, sleepUntil.toSystemTimePoint());
+            sleepUntil = lastInvalidationTime + Seconds(userCacheInvalidationIntervalSecs.load());
             now = Date_t::now();
         }
         lastInvalidationTime = now;
@@ -160,11 +163,11 @@ void UserCacheInvalidator::run() {
             if (currentGeneration.getStatus().code() == ErrorCodes::CommandNotFound) {
                 warning() << "_getUserCacheGeneration command not found on config server(s), "
                              "this most likely means you are running an outdated version of mongod "
-                             "on the config servers" << std::endl;
+                             "on the config servers";
             } else {
                 warning() << "An error occurred while fetching current user cache generation "
                              "to check if user cache needs invalidation: "
-                          << currentGeneration.getStatus() << std::endl;
+                          << currentGeneration.getStatus();
             }
             // When in doubt, invalidate the cache
             _authzManager->invalidateUserCache();
@@ -173,7 +176,7 @@ void UserCacheInvalidator::run() {
 
         if (currentGeneration.getValue() != _previousCacheGeneration) {
             log() << "User cache generation changed from " << _previousCacheGeneration << " to "
-                  << currentGeneration.getValue() << "; invalidating user cache" << std::endl;
+                  << currentGeneration.getValue() << "; invalidating user cache";
             _authzManager->invalidateUserCache();
             _previousCacheGeneration = currentGeneration.getValue();
         }
