@@ -36,31 +36,36 @@
 
 #include <algorithm>
 #include <boost/filesystem/operations.hpp>
+#include <memory>
 
 #include "mongo/db/audit.h"
 #include "mongo/db/auth/auth_index_d.h"
 #include "mongo/db/background.h"
-#include "mongo/db/clientcursor.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/collection_catalog_entry.h"
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/database_catalog_entry.h"
 #include "mongo/db/catalog/database_holder.h"
+#include "mongo/db/clientcursor.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/service_context.h"
-#include "mongo/db/service_context_d.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/instance.h"
 #include "mongo/db/introspect.h"
 #include "mongo/db/op_observer.h"
+#include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/replication_coordinator_global.h"
+#include "mongo/db/server_options.h"
 #include "mongo/db/server_parameters.h"
+#include "mongo/db/service_context.h"
+#include "mongo/db/service_context_d.h"
 #include "mongo/db/stats/top.h"
-#include "mongo/db/storage/storage_options.h"
-#include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/recovery_unit.h"
+#include "mongo/db/storage/storage_engine.h"
+#include "mongo/db/storage/storage_options.h"
+#include "mongo/db/views/view_catalog.h"
+#include "mongo/util/assert_util.h"
 #include "mongo/util/log.h"
 
 namespace mongo {
@@ -141,7 +146,7 @@ void Database::close(OperationContext* txn) {
     repl::oplogCheckCloseDatabase(txn, this);
 
     if (BackgroundOperation::inProgForDb(_name)) {
-        log() << "warning: bg op in prog during close db? " << _name << endl;
+        log() << "warning: bg op in prog during close db? " << _name;
     }
 }
 
@@ -198,10 +203,13 @@ Database::Database(OperationContext* txn, StringData name, DatabaseCatalogEntry*
     : _name(name.toString()),
       _dbEntry(dbEntry),
       _profileName(_name + ".system.profile"),
-      _indexesName(_name + ".system.indexes") {
+      _indexesName(_name + ".system.indexes"),
+      _viewsName(_name + "." + DurableViewCatalog::viewsCollectionName().toString()),
+      _durableViews(DurableViewCatalogImpl(this)),
+      _views(&_durableViews) {
     Status status = validateDBName(_name);
     if (!status.isOK()) {
-        warning() << "tried to open invalid db: " << _name << endl;
+        warning() << "tried to open invalid db: " << _name;
         uasserted(10028, status.toString());
     }
 
@@ -213,9 +221,19 @@ Database::Database(OperationContext* txn, StringData name, DatabaseCatalogEntry*
         const string ns = *it;
         _collections[ns] = _getOrCreateCollectionInstance(txn, ns);
     }
+    // At construction time of the viewCatalog, the _collections map wasn't initialized yet, so no
+    // system.views collection would be found. Now we're sufficiently initialized, signal a version
+    // change. Also force a reload, so if there are problems with the catalog contents as might be
+    // caused by incorrect mongod versions or similar, they are found right away.
+    _views.invalidate();
+    Status reloadStatus = _views.reloadIfNeeded(txn);
+    if (!reloadStatus.isOK()) {
+        warning() << "Unable to parse views: " << redact(reloadStatus)
+                  << "; remove any invalid views from the " << _viewsName
+                  << " collection to restore server functionality." << startupWarningsLog;
+    }
 }
 
-
 /*static*/
 string Database::duplicateUncasedName(const string& name, set<string>* duplicates) {
     if (duplicates) {
@@ -263,7 +281,7 @@ void Database::clearTmpCollections(OperationContext* txn) {
             WriteUnitOfWork wunit(txn);
             Status status = dropCollection(txn, ns);
             if (!status.isOK()) {
-                warning() << "could not drop temp collection '" << ns << "': " << status;
+                warning() << "could not drop temp collection '" << ns << "': " << redact(status);
                 continue;
             }
 
@@ -304,7 +322,8 @@ void Database::getStats(OperationContext* opCtx, BSONObjBuilder* output, double
     list<string> collections;
     _dbEntry->getCollectionNamespaces(&collections);
 
-    long long ncollections = 0;
+    long long nCollections = 0;
+    long long nViews = 0;
     long long objects = 0;
     long long size = 0;
     long long storageSize = 0;
@@ -319,7 +338,7 @@ void Database::getStats(OperationContext* opCtx, BSONObjBuilder* output, double
         if (!collection)
             continue;
 
-        ncollections += 1;
+        nCollections += 1;
         objects += collection->numRecords(opCtx);
         size += collection->dataSize(opCtx);
 
@@ -331,7 +350,10 @@ void Database::getStats(OperationContext* opCtx, BSONObjBuilder* output, double
         indexSize += collection->getIndexSize(opCtx);
     }
 
-    output->appendNumber("collections", ncollections);
+    getViewCatalog()->iterate(opCtx, [&](const ViewDefinition& view) { nViews += 1; });
+
+    output->appendNumber("collections", nCollections);
+    output->appendNumber("views", nViews);
     output->appendNumber("objects", objects);
     output->append("avgObjSize", objects == 0 ? 0 : double(size) / double(objects));
     output->appendNumber("dataSize", size / scale);
@@ -343,10 +365,16 @@ void Database::getStats(OperationContext* opCtx, BSONObjBuilder* output, double
     _dbEntry->appendExtraStats(opCtx, output, scale);
 }
 
+Status Database::dropView(OperationContext* txn, StringData fullns) {
+    Status status = _views.dropView(txn, NamespaceString(fullns));
+    Top::get(txn->getClient()->getServiceContext()).collectionDropped(fullns);
+    return status;
+}
+
 Status Database::dropCollection(OperationContext* txn, StringData fullns) {
     invariant(txn->lockState()->isDbLockedForMode(name(), MODE_X));
 
-    LOG(1) << "dropCollection: " << fullns << endl;
+    LOG(1) << "dropCollection: " << fullns;
     massertNamespaceNotIndex(fullns, "dropCollection");
 
     Collection* collection = getCollection(fullns);
@@ -364,7 +392,7 @@ Status Database::dropCollection(OperationContext* txn, StringData fullns) {
                 if (_profile != 0)
                     return Status(ErrorCodes::IllegalOperation,
                                   "turn off profiling before dropping system.profile collection");
-            } else {
+            } else if (!nss.isSystemDotViews()) {
                 return Status(ErrorCodes::IllegalOperation, "can't drop system ns");
             }
         }
@@ -377,12 +405,12 @@ Status Database::dropCollection(OperationContext* txn, StringData fullns) {
     Status s = collection->getIndexCatalog()->dropAllIndexes(txn, true);
     if (!s.isOK()) {
         warning() << "could not drop collection, trying to drop indexes" << fullns << " because of "
-                  << s.toString();
+                  << redact(s.toString());
         return s;
     }
 
     verify(collection->_details->getTotalIndexCount(txn) == 0);
-    LOG(1) << "\t dropIndexes done" << endl;
+    LOG(1) << "\t dropIndexes done";
 
     Top::get(txn->getClient()->getServiceContext()).collectionDropped(fullns);
 
@@ -407,7 +435,10 @@ Status Database::dropCollection(OperationContext* txn, StringData fullns) {
         }
     }
 
-    getGlobalServiceContext()->getOpObserver()->onDropCollection(txn, nss);
+    auto opObserver = getGlobalServiceContext()->getOpObserver();
+    if (opObserver)
+        opObserver->onDropCollection(txn, nss);
+
     return Status::OK();
 }
 
@@ -436,7 +467,6 @@ Collection* Database::getCollection(StringData ns) const {
     return NULL;
 }
 
-
 Status Database::renameCollection(OperationContext* txn,
                                   StringData fromNS,
                                   StringData toNS,
@@ -479,31 +509,56 @@ Collection* Database::getOrCreateCollection(OperationContext* txn, StringData ns
     return c;
 }
 
+void Database::_checkCanCreateCollection(const NamespaceString& nss,
+                                         const CollectionOptions& options) {
+    massert(17399, "collection already exists", getCollection(nss.ns()) == nullptr);
+    massertNamespaceNotIndex(nss.ns(), "createCollection");
+
+    uassert(14037,
+            "can't create user databases on a --configsvr instance",
+            serverGlobalParams.clusterRole != ClusterRole::ConfigServer || nss.isOnInternalDb());
+
+    // This check only applies for actual collections, not indexes or other types of ns.
+    uassert(17381,
+            str::stream() << "fully qualified namespace " << nss.ns() << " is too long "
+                          << "(max is "
+                          << NamespaceString::MaxNsCollectionLen
+                          << " bytes)",
+            !nss.isNormal() || nss.size() <= NamespaceString::MaxNsCollectionLen);
+
+    uassert(17316, "cannot create a blank collection", nss.coll() > 0);
+    uassert(28838, "cannot create a non-capped oplog collection", options.capped || !nss.isOplog());
+}
+
+Status Database::createView(OperationContext* txn,
+                            StringData ns,
+                            const CollectionOptions& options) {
+    invariant(txn->lockState()->isDbLockedForMode(name(), MODE_X));
+    invariant(options.isView());
+
+    NamespaceString nss(ns);
+    NamespaceString viewOnNss(nss.db(), options.viewOn);
+    _checkCanCreateCollection(nss, options);
+    audit::logCreateCollection(&cc(), ns);
+
+    if (nss.isOplog())
+        return Status(ErrorCodes::InvalidNamespace,
+                      str::stream() << "invalid namespace name for a view: " + nss.toString());
+
+    return _views.createView(txn, nss, viewOnNss, BSONArray(options.pipeline), options.collation);
+}
+
+
 Collection* Database::createCollection(OperationContext* txn,
                                        StringData ns,
                                        const CollectionOptions& options,
-                                       bool createIdIndex) {
-    massert(17399, "collection already exists", getCollection(ns) == NULL);
-    massertNamespaceNotIndex(ns, "createCollection");
+                                       bool createIdIndex,
+                                       const BSONObj& idIndex) {
     invariant(txn->lockState()->isDbLockedForMode(name(), MODE_X));
-
-    if (serverGlobalParams.configsvr &&
-        !(ns.startsWith("config.") || ns.startsWith("local.") || ns.startsWith("admin."))) {
-        uasserted(14037, "can't create user databases on a --configsvr instance");
-    }
-
-    if (NamespaceString::normal(ns)) {
-        // This check only applies for actual collections, not indexes or other types of ns.
-        uassert(17381,
-                str::stream() << "fully qualified namespace " << ns << " is too long "
-                              << "(max is " << NamespaceString::MaxNsCollectionLen << " bytes)",
-                ns.size() <= NamespaceString::MaxNsCollectionLen);
-    }
+    invariant(!options.isView());
 
     NamespaceString nss(ns);
-    uassert(17316, "cannot create a blank collection", nss.coll() > 0);
-    uassert(28838, "cannot create a non-capped oplog collection", options.capped || !nss.isOplog());
-
+    _checkCanCreateCollection(nss, options);
     audit::logCreateCollection(&cc(), ns);
 
     Status status = _dbEntry->createCollection(txn, ns, options, true /*allocateDefaultSpace*/);
@@ -514,12 +569,19 @@ Collection* Database::createCollection(OperationContext* txn,
     invariant(collection);
     _collections[ns] = collection;
 
+    BSONObj fullIdIndexSpec;
+
     if (createIdIndex) {
         if (collection->requiresIdIndex()) {
             if (options.autoIndexId == CollectionOptions::YES ||
                 options.autoIndexId == CollectionOptions::DEFAULT) {
+                const auto featureCompatibilityVersion =
+                    serverGlobalParams.featureCompatibility.version.load();
                 IndexCatalog* ic = collection->getIndexCatalog();
-                uassertStatusOK(ic->createIndexOnEmptyCollection(txn, ic->getDefaultIdIndexSpec()));
+                fullIdIndexSpec = uassertStatusOK(ic->createIndexOnEmptyCollection(
+                    txn,
+                    !idIndex.isEmpty() ? idIndex
+                                       : ic->getDefaultIdIndexSpec(featureCompatibilityVersion)));
             }
         }
 
@@ -528,7 +590,9 @@ Collection* Database::createCollection(OperationContext* txn,
         }
     }
 
-    getGlobalServiceContext()->getOpObserver()->onCreateCollection(txn, nss, options);
+    auto opObserver = getGlobalServiceContext()->getOpObserver();
+    if (opObserver)
+        opObserver->onCreateCollection(txn, nss, options, fullIdIndexSpec);
 
     return collection;
 }
@@ -547,7 +611,7 @@ void dropAllDatabasesExceptLocal(OperationContext* txn) {
 
     if (n.size() == 0)
         return;
-    log() << "dropAllDatabasesExceptLocal " << n.size() << endl;
+    log() << "dropAllDatabasesExceptLocal " << n.size();
 
     repl::getGlobalReplicationCoordinator()->dropAllSnapshots();
     for (vector<string>::iterator i = n.begin(); i != n.end(); i++) {
@@ -559,7 +623,7 @@ void dropAllDatabasesExceptLocal(OperationContext* txn) {
                 if (db == nullptr) {
                     log() << "database disappeared after listDatabases but before drop: " << *i;
                 } else {
-                    dropDatabase(txn, db);
+                    Database::dropDatabase(txn, db);
                 }
             }
             MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, "dropAllDatabasesExceptLocal", *i);
@@ -567,34 +631,38 @@ void dropAllDatabasesExceptLocal(OperationContext* txn) {
     }
 }
 
-void dropDatabase(OperationContext* txn, Database* db) {
+void Database::dropDatabase(OperationContext* txn, Database* db) {
     invariant(db);
 
     // Store the name so we have if for after the db object is deleted
     const string name = db->name();
-    LOG(1) << "dropDatabase " << name << endl;
+    LOG(1) << "dropDatabase " << name;
 
     invariant(txn->lockState()->isDbLockedForMode(name, MODE_X));
 
-    BackgroundOperation::assertNoBgOpInProgForDb(name.c_str());
+    BackgroundOperation::assertNoBgOpInProgForDb(name);
 
-    audit::logDropDatabase(&cc(), name);
+    audit::logDropDatabase(txn->getClient(), name);
+
+    for (auto&& coll : *db) {
+        Top::get(txn->getClient()->getServiceContext()).collectionDropped(coll->ns().ns(), true);
+    }
 
     dbHolder().close(txn, name);
     db = NULL;  // d is now deleted
 
-    getGlobalServiceContext()->getGlobalStorageEngine()->dropDatabase(txn, name);
+    MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
+        getGlobalServiceContext()->getGlobalStorageEngine()->dropDatabase(txn, name);
+    }
+    MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, "dropDatabase", name);
 }
 
-/** { ..., capped: true, size: ..., max: ... }
- * @param createDefaultIndexes - if false, defers id (and other) index creation.
- * @return true if successful
-*/
 Status userCreateNS(OperationContext* txn,
                     Database* db,
                     StringData ns,
                     BSONObj options,
-                    bool createDefaultIndexes) {
+                    bool createDefaultIndexes,
+                    const BSONObj& idIndex) {
     invariant(db);
 
     LOG(1) << "create collection " << ns << ' ' << options;
@@ -605,13 +673,38 @@ Status userCreateNS(OperationContext* txn,
     Collection* collection = db->getCollection(ns);
 
     if (collection)
-        return Status(ErrorCodes::NamespaceExists, "collection already exists");
+        return Status(ErrorCodes::NamespaceExists,
+                      str::stream() << "a collection '" << ns.toString() << "' already exists");
+
+    if (db->getViewCatalog()->lookup(txn, ns))
+        return Status(ErrorCodes::NamespaceExists,
+                      str::stream() << "a view '" << ns.toString() << "' already exists");
 
     CollectionOptions collectionOptions;
     Status status = collectionOptions.parse(options);
     if (!status.isOK())
         return status;
 
+    // Validate the collation, if there is one.
+    if (!collectionOptions.collation.isEmpty()) {
+        auto collator = CollatorFactoryInterface::get(txn->getServiceContext())
+                            ->makeFromBSON(collectionOptions.collation);
+        if (!collator.isOK()) {
+            return collator.getStatus();
+        }
+
+        // If the collator factory returned a non-null collator, set the collation option to the
+        // result of serializing the collator's spec back into BSON. We do this in order to fill in
+        // all options that the user omitted.
+        //
+        // If the collator factory returned a null collator (representing the "simple" collation),
+        // we simply unset the "collation" from the collection options. This ensures that
+        // collections created on versions which do not support the collation feature have the same
+        // format for representing the simple collation as collections created on this version.
+        collectionOptions.collation =
+            collator.getValue() ? collator.getValue()->getSpec().toBSON() : BSONObj();
+    }
+
     status =
         validateStorageOptions(collectionOptions.storageEngine,
                                stdx::bind(&StorageEngine::Factory::validateCollectionStorageOptions,
@@ -631,7 +724,11 @@ Status userCreateNS(OperationContext* txn,
         }
     }
 
-    invariant(db->createCollection(txn, ns, collectionOptions, createDefaultIndexes));
+    if (collectionOptions.isView()) {
+        uassertStatusOK(db->createView(txn, ns, collectionOptions));
+    } else {
+        invariant(db->createCollection(txn, ns, collectionOptions, createDefaultIndexes, idIndex));
+    }
 
     return Status::OK();
 }
