@@ -30,15 +30,20 @@
 
 #pragma once
 
+#include <boost/optional.hpp>
 #include <string>
 
 #include "mongo/base/owned_pointer_vector.h"
 #include "mongo/bson/ordering.h"
+#include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/storage/snapshot_name.h"
+#include "mongo/platform/atomic_word.h"
+#include "mongo/stdx/mutex.h"
 
 namespace mongo {
 
+class CollatorInterface;
 class CollectionCatalogEntry;
 class CollectionInfoCache;
 class HeadManager;
@@ -51,7 +56,8 @@ class IndexCatalogEntry {
     MONGO_DISALLOW_COPYING(IndexCatalogEntry);
 
 public:
-    IndexCatalogEntry(StringData ns,
+    IndexCatalogEntry(OperationContext* txn,
+                      StringData ns,
                       CollectionCatalogEntry* collection,  // not owned
                       IndexDescriptor* descriptor,         // ownership passes to me
                       CollectionInfoCache* infoCache);     // not owned, optional
@@ -62,7 +68,7 @@ public:
         return _ns;
     }
 
-    void init(OperationContext* txn, IndexAccessMethod* accessMethod);
+    void init(std::unique_ptr<IndexAccessMethod> accessMethod);
 
     IndexDescriptor* descriptor() {
         return _descriptor;
@@ -72,10 +78,10 @@ public:
     }
 
     IndexAccessMethod* accessMethod() {
-        return _accessMethod;
+        return _accessMethod.get();
     }
     const IndexAccessMethod* accessMethod() const {
-        return _accessMethod;
+        return _accessMethod.get();
     }
 
     const Ordering& ordering() const {
@@ -86,6 +92,10 @@ public:
         return _filterExpression.get();
     }
 
+    const CollatorInterface* getCollator() const {
+        return _collator.get();
+    }
+
     /// ---------------------
 
     const RecordId& head(OperationContext* txn) const;
@@ -100,9 +110,33 @@ public:
 
     // --
 
+    /**
+     * Returns true if this index is multikey, and returns false otherwise.
+     */
     bool isMultikey() const;
 
-    void setMultikey(OperationContext* txn);
+    /**
+     * Returns the path components that cause this index to be multikey if this index supports
+     * path-level multikey tracking, and returns an empty vector if path-level multikey tracking
+     * isn't supported.
+     *
+     * If this index supports path-level multikey tracking but isn't multikey, then this function
+     * returns a vector with size equal to the number of elements in the index key pattern where
+     * each element in the vector is an empty set.
+     */
+    MultikeyPaths getMultikeyPaths(OperationContext* txn) const;
+
+    /**
+     * Sets this index to be multikey. Information regarding which newly detected path components
+     * cause this index to be multikey can also be specified.
+     *
+     * If this index doesn't support path-level multikey tracking, then 'multikeyPaths' is ignored.
+     *
+     * If this index supports path-level multikey tracking, then 'multikeyPaths' must be a vector
+     * with size equal to the number of elements in the index key pattern. Additionally, at least
+     * one path component of the indexed fields must cause this index to be multikey.
+     */
+    void setMultikey(OperationContext* txn, const MultikeyPaths& multikeyPaths);
 
     // if this ready is ready for queries
     bool isReady(OperationContext* txn) const;
@@ -125,7 +159,13 @@ private:
 
     bool _catalogIsReady(OperationContext* txn) const;
     RecordId _catalogHead(OperationContext* txn) const;
-    bool _catalogIsMultikey(OperationContext* txn) const;
+
+    /**
+     * Retrieves the multikey information associated with this index from '_collection',
+     *
+     * See CollectionCatalogEntry::isIndexMultikey() for more details.
+     */
+    bool _catalogIsMultikey(OperationContext* txn, MultikeyPaths* multikeyPaths) const;
 
     // -----
 
@@ -137,10 +177,11 @@ private:
 
     CollectionInfoCache* _infoCache;  // not owned here
 
-    IndexAccessMethod* _accessMethod;  // owned here
+    std::unique_ptr<IndexAccessMethod> _accessMethod;
 
     // Owned here.
     HeadManager* _headManager;
+    std::unique_ptr<CollatorInterface> _collator;
     std::unique_ptr<MatchExpression> _filterExpression;
 
     // cached stuff
@@ -148,7 +189,28 @@ private:
     Ordering _ordering;  // TODO: this might be b-tree specific
     bool _isReady;       // cache of NamespaceDetails info
     RecordId _head;      // cache of IndexDetails
-    bool _isMultikey;    // cache of NamespaceDetails info
+
+    // Set to true if this index supports path-level multikey tracking.
+    // '_indexTracksPathLevelMultikeyInfo' is effectively const after IndexCatalogEntry::init() is
+    // called.
+    bool _indexTracksPathLevelMultikeyInfo = false;
+
+    // Set to true if this index is multikey. '_isMultikey' serves as a cache of the information
+    // stored in the NamespaceDetails or KVCatalog.
+    AtomicWord<bool> _isMultikey;
+
+    // Controls concurrent access to '_indexMultikeyPaths'. We acquire this mutex rather than the
+    // RESOURCE_METADATA lock as a performance optimization so that it is cheaper to detect whether
+    // there is actually any path-level multikey information to update or not.
+    mutable stdx::mutex _indexMultikeyPathsMutex;
+
+    // Non-empty only if '_indexTracksPathLevelMultikeyInfo' is true.
+    //
+    // If non-empty, '_indexMultikeyPaths' is a vector with size equal to the number of elements
+    // in the index key pattern. Each element in the vector is an ordered set of positions (starting
+    // at 0) into the corresponding indexed field that represent what prefixes of the indexed field
+    // causes the index to be multikey.
+    MultikeyPaths _indexMultikeyPaths;
 
     // The earliest snapshot that is allowed to read this index.
     boost::optional<SnapshotName> _minVisibleSnapshot;
