@@ -33,6 +33,7 @@
 #include "mongo/db/ops/field_checker.h"
 #include "mongo/db/ops/log_builder.h"
 #include "mongo/db/ops/path_support.h"
+#include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/util/mongoutils/str.h"
 
 namespace mongo {
@@ -124,7 +125,8 @@ Status ModifierAddToSet::init(const BSONElement& modExpr, const Options& opts, b
     if (foundDollar && foundCount > 1) {
         return Status(ErrorCodes::BadValue,
                       str::stream() << "Too many positional (i.e. '$') elements found in path '"
-                                    << _fieldRef.dottedField() << "'");
+                                    << _fieldRef.dottedField()
+                                    << "'");
     }
 
     // TODO: The driver could potentially do this re-writing.
@@ -148,8 +150,6 @@ Status ModifierAddToSet::init(const BSONElement& modExpr, const Options& opts, b
                 return status;
 
             _val = _valDoc.root().leftChild();
-
-            deduplicate(_val, mb::woLess(false), mb::woEqual(false));
         }
     }
 
@@ -197,9 +197,17 @@ Status ModifierAddToSet::init(const BSONElement& modExpr, const Options& opts, b
         valCursor = valCursor.rightSibling();
     }
 
+    setCollator(opts.collator);
     return Status::OK();
 }
 
+void ModifierAddToSet::setCollator(const CollatorInterface* collator) {
+    invariant(!_collator);
+    _collator = collator;
+    // Deduplicate _val (must be performed after collator is set to final value.)
+    deduplicate(_val, mb::woLess(_collator, false), mb::woEqual(_collator, false));
+}
+
 Status ModifierAddToSet::prepare(mb::Element root, StringData matchedField, ExecInfo* execInfo) {
     _preparedState.reset(new PreparedState(root.getDocument()));
 
@@ -247,11 +255,12 @@ Status ModifierAddToSet::prepare(mb::Element root, StringData matchedField, Exec
     if (_preparedState->elemFound.getType() != mongo::Array) {
         mb::Element idElem = mb::findElementNamed(root.leftChild(), "_id");
         return Status(ErrorCodes::BadValue,
-                      str::stream()
-                          << "Cannot apply $addToSet to a non-array field. Field named '"
-                          << _preparedState->elemFound.getFieldName() << "' has a non-array type "
-                          << typeName(_preparedState->elemFound.getType()) << " in the document "
-                          << idElem.toString());
+                      str::stream() << "Cannot apply $addToSet to a non-array field. Field named '"
+                                    << _preparedState->elemFound.getFieldName()
+                                    << "' has a non-array type "
+                                    << typeName(_preparedState->elemFound.getType())
+                                    << " in the document "
+                                    << idElem.toString());
     }
 
     // If the array is empty, then we don't need to check anything: all of the values are
@@ -265,8 +274,8 @@ Status ModifierAddToSet::prepare(mb::Element root, StringData matchedField, Exec
     // the element is not present, record it as one to add.
     mb::Element eachIter = _val.leftChild();
     while (eachIter.ok()) {
-        mb::Element where =
-            mb::findElement(_preparedState->elemFound.leftChild(), mb::woEqualTo(eachIter, false));
+        mb::Element where = mb::findElement(_preparedState->elemFound.leftChild(),
+                                            mb::woEqualTo(eachIter, _collator, false));
         if (!where.ok()) {
             // The element was not found. Record the element from $each as one to be added.
             _preparedState->elementsToAdd.push_back(eachIter);
@@ -387,7 +396,8 @@ Status ModifierAddToSet::log(LogBuilder* logBuilder) const {
         if (!status.isOK()) {
             return Status(ErrorCodes::BadValue,
                           str::stream() << "Could not append entry for $addToSet oplog entry."
-                                        << "Underlying cause: " << status.toString());
+                                        << "Underlying cause: "
+                                        << status.toString());
         }
         curr = curr.rightSibling();
     }
