@@ -35,13 +35,15 @@
 
 #include "mongo/base/status.h"
 #include "mongo/base/string_data.h"
-#include "mongo/bson/ordering.h"
 #include "mongo/bson/mutable/algorithm.h"
 #include "mongo/bson/mutable/document.h"
 #include "mongo/bson/mutable/mutable_bson_test_utils.h"
+#include "mongo/bson/ordering.h"
+#include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/json.h"
 #include "mongo/db/ops/log_builder.h"
+#include "mongo/db/query/collation/collator_interface_mock.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/mongoutils/str.h"
 
@@ -50,6 +52,7 @@ namespace {
 using mongo::BSONObj;
 using mongo::BSONObjBuilder;
 using mongo::BSONArrayBuilder;
+using mongo::CollatorInterfaceMock;
 using mongo::fromjson;
 using mongo::LogBuilder;
 using mongo::ModifierInterface;
@@ -65,6 +68,8 @@ using mongo::mutablebson::Element;
 using std::sort;
 using std::vector;
 
+namespace dps = ::mongo::dotted_path_support;
+
 void combineVec(const vector<int>& origVec,
                 const vector<int>& modVec,
                 int32_t slice,
@@ -113,8 +118,8 @@ struct ProjectKeyCmp {
         if (useWholeValue) {
             ret = left.woCompare(right, Ordering::make(sortPattern), false);
         } else {
-            BSONObj lhsKey = left.extractFields(sortPattern, true);
-            BSONObj rhsKey = right.extractFields(sortPattern, true);
+            BSONObj lhsKey = dps::extractElementsBasedOnTemplate(left, sortPattern, true);
+            BSONObj rhsKey = dps::extractElementsBasedOnTemplate(right, sortPattern, true);
             ret = lhsKey.woCompare(rhsKey, sortPattern);
         }
         return ret < 0;
@@ -481,14 +486,14 @@ class Mod {
 public:
     Mod() : _mod() {}
 
-    explicit Mod(BSONObj modObj)
+    explicit Mod(BSONObj modObj,
+                 ModifierInterface::Options options = ModifierInterface::Options::normal())
         : _mod(mongoutils::str::equals(modObj.firstElement().fieldName(), "$pushAll")
                    ? ModifierPush::PUSH_ALL
                    : ModifierPush::PUSH_NORMAL) {
         _modObj = modObj;
         StringData modName = modObj.firstElement().fieldName();
-        ASSERT_OK(_mod.init(_modObj[modName].embeddedObject().firstElement(),
-                            ModifierInterface::Options::normal()));
+        ASSERT_OK(_mod.init(_modObj[modName].embeddedObject().firstElement(), options));
     }
 
     Status prepare(Element root, StringData matchedField, ModifierInterface::ExecInfo* execInfo) {
@@ -659,13 +664,13 @@ TEST(SimpleObjMod, PrepareApplyNormal) {
 }
 
 TEST(SimpleObjMod, PrepareApplyDotted) {
-    Document doc(fromjson(
-        "{ _id : 1 , "
-        "  question : 'a', "
-        "  choices : { "
-        "            first : { choice : 'b' }, "
-        "            second : { choice : 'c' } }"
-        "}"));
+    Document doc(
+        fromjson("{ _id : 1 , "
+                 "  question : 'a', "
+                 "  choices : { "
+                 "            first : { choice : 'b' }, "
+                 "            second : { choice : 'c' } }"
+                 "}"));
     Mod pushMod(fromjson("{$push: {'choices.first.votes': 1}}"));
 
     ModifierInterface::ExecInfo execInfo;
@@ -676,13 +681,12 @@ TEST(SimpleObjMod, PrepareApplyDotted) {
 
     ASSERT_OK(pushMod.apply());
     ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(fromjson(
-                      "{ _id : 1 , "
-                      "  question : 'a', "
-                      "  choices : { "
-                      "            first : { choice : 'b', votes: [1]}, "
-                      "            second : { choice : 'c' } }"
-                      "}"),
+    ASSERT_EQUALS(fromjson("{ _id : 1 , "
+                           "  question : 'a', "
+                           "  choices : { "
+                           "            first : { choice : 'b', votes: [1]}, "
+                           "            second : { choice : 'c' } }"
+                           "}"),
                   doc);
 
     Document logDoc;
@@ -1012,6 +1016,36 @@ TEST(SortPushEach, MixedSortEmbeddedField) {
     ASSERT_EQUALS(BSON("$set" << expectedObj), logDoc);
 }
 
+TEST(SortPushEach, SortRespectsCollationFromOptions) {
+    Document doc(fromjson("{a: ['dd', 'fc', 'gb'] }"));
+    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
+    Mod pushMod(fromjson("{$push: {a: {$each: ['ha'], $sort: 1}}}"),
+                ModifierInterface::Options::normal(&collator));
+    const BSONObj expectedObj = fromjson("{a: ['ha', 'gb', 'fc', 'dd']}");
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_EQUALS(expectedObj, doc);
+}
+
+TEST(SortPushEach, SortRespectsCollationFromSetCollator) {
+    Document doc(fromjson("{a: ['dd', 'fc', 'gb'] }"));
+    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
+    Mod pushMod(fromjson("{$push: {a: {$each: ['ha'], $sort: 1}}}"));
+    pushMod.mod().setCollator(&collator);
+    const BSONObj expectedObj = fromjson("{a: ['ha', 'gb', 'fc', 'dd']}");
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_EQUALS(expectedObj, doc);
+}
+
 /**
  * This fixture supports building $push mods with parameterized $each arrays and $slices.
  * It always assume that the array being operated on is called 'a'. To build a mod, one
@@ -1059,8 +1093,9 @@ public:
             arrBuilder.append(*it);
         }
 
-        _modObj = BSON("$push" << BSON("a" << BSON("$each" << arrBuilder.arr() << "$slice" << slice
-                                                           << "$sort" << sort)));
+        _modObj = BSON(
+            "$push" << BSON(
+                "a" << BSON("$each" << arrBuilder.arr() << "$slice" << slice << "$sort" << sort)));
 
         ASSERT_OK(_mod.init(_modObj["$push"].embeddedObject().firstElement(),
                             ModifierInterface::Options::normal()));
@@ -1279,8 +1314,9 @@ TEST(ToPosition, BadInputs) {
         "{$push: {a: { $each: [1], $position:3.000000000001}}}",
         "{$push: {a: { $each: [1], $position:1.2}}}",
         "{$push: {a: { $each: [1], $position:-1.2}}}",
-        "{$push: {a: { $each: [1], $position:9.0e19}}}",
-        "{$push: {a: { $each: [1], $position:-9.0e19}}}",
+        "{$push: {a: { $each: [1], $position:2147483648}}}",
+        "{$push: {a: { $each: [1], $position:-2147483649}}}",
+        "{$push: {a: { $each: [1], $position:NaN}}}",
         NULL,
     };
 
