@@ -26,52 +26,62 @@
  * it in the license file.
  */
 
-#include "mongo/pch.h"
+#include "mongo/platform/basic.h"
 
 #include "mongo/db/pipeline/accumulator.h"
+
+#include "mongo/db/pipeline/accumulation_statement.h"
+#include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/value.h"
 
 namespace mongo {
 
-    void AccumulatorMinMax::processInternal(const Value& input, bool merging) {
-        // nullish values should have no impact on result
-        if (!input.nullish()) {
-            /* compare with the current value; swap if appropriate */
-            int cmp = Value::compare(_val, input) * _sense;
-            if (cmp > 0 || _val.missing()) { // missing is lower than all other values
-                _val = input;
-                _memUsageBytes = sizeof(*this) + input.getApproximateSize() - sizeof(Value);
-            }
+using boost::intrusive_ptr;
+
+REGISTER_ACCUMULATOR(max, AccumulatorMax::create);
+REGISTER_ACCUMULATOR(min, AccumulatorMin::create);
+REGISTER_EXPRESSION(max, ExpressionFromAccumulator<AccumulatorMax>::parse);
+REGISTER_EXPRESSION(min, ExpressionFromAccumulator<AccumulatorMin>::parse);
+
+const char* AccumulatorMinMax::getOpName() const {
+    if (_sense == 1)
+        return "$min";
+    return "$max";
+}
+
+void AccumulatorMinMax::processInternal(const Value& input, bool merging) {
+    // nullish values should have no impact on result
+    if (!input.nullish()) {
+        /* compare with the current value; swap if appropriate */
+        int cmp = getExpressionContext()->getValueComparator().compare(_val, input) * _sense;
+        if (cmp > 0 || _val.missing()) {  // missing is lower than all other values
+            _val = input;
+            _memUsageBytes = sizeof(*this) + input.getApproximateSize() - sizeof(Value);
         }
     }
+}
 
-    Value AccumulatorMinMax::getValue(bool toBeMerged) const {
-        return _val;
+Value AccumulatorMinMax::getValue(bool toBeMerged) const {
+    if (_val.missing()) {
+        return Value(BSONNULL);
     }
+    return _val;
+}
 
-    AccumulatorMinMax::AccumulatorMinMax(int theSense)
-        :_sense(theSense)
-    {
-        verify((_sense == 1) || (_sense == -1));
-        _memUsageBytes = sizeof(*this);
-    }
-
-    void AccumulatorMinMax::reset() {
-        _val = Value();
-        _memUsageBytes = sizeof(*this);
-    }
+AccumulatorMinMax::AccumulatorMinMax(Sense sense) : _sense(sense) {
+    _memUsageBytes = sizeof(*this);
+}
 
-    intrusive_ptr<Accumulator> AccumulatorMinMax::createMin() {
-        return new AccumulatorMinMax(1);
-    }
+void AccumulatorMinMax::reset() {
+    _val = Value();
+    _memUsageBytes = sizeof(*this);
+}
 
-    intrusive_ptr<Accumulator> AccumulatorMinMax::createMax() {
-        return new AccumulatorMinMax(-1);
-    }
+intrusive_ptr<Accumulator> AccumulatorMin::create() {
+    return new AccumulatorMin();
+}
 
-    const char *AccumulatorMinMax::getOpName() const {
-        if (_sense == 1)
-            return "$min";
-        return "$max";
-    }
+intrusive_ptr<Accumulator> AccumulatorMax::create() {
+    return new AccumulatorMax();
+}
 }
