@@ -62,7 +62,8 @@ TEST(MemberConfig, ParseFailsWithIllegalFieldName) {
     ASSERT_EQUALS(ErrorCodes::BadValue,
                   mc.initialize(BSON("_id" << 0 << "host"
                                            << "localhost"
-                                           << "frim" << 1),
+                                           << "frim"
+                                           << 1),
                                 &tagConfig));
 }
 
@@ -121,12 +122,14 @@ TEST(MemberConfig, ParseArbiterOnly) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "arbiterOnly" << 1.0),
+                                       << "arbiterOnly"
+                                       << 1.0),
                             &tagConfig));
     ASSERT_TRUE(mc.isArbiter());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "arbiterOnly" << false),
+                                       << "arbiterOnly"
+                                       << false),
                             &tagConfig));
     ASSERT_TRUE(!mc.isArbiter());
 }
@@ -136,12 +139,14 @@ TEST(MemberConfig, ParseHidden) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "hidden" << 1.0),
+                                       << "hidden"
+                                       << 1.0),
                             &tagConfig));
     ASSERT_TRUE(mc.isHidden());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "hidden" << false),
+                                       << "hidden"
+                                       << false),
                             &tagConfig));
     ASSERT_TRUE(!mc.isHidden());
     ASSERT_EQUALS(ErrorCodes::TypeMismatch,
@@ -157,12 +162,14 @@ TEST(MemberConfig, ParseBuildIndexes) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "buildIndexes" << 1.0),
+                                       << "buildIndexes"
+                                       << 1.0),
                             &tagConfig));
     ASSERT_TRUE(mc.shouldBuildIndexes());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "buildIndexes" << false),
+                                       << "buildIndexes"
+                                       << false),
                             &tagConfig));
     ASSERT_TRUE(!mc.shouldBuildIndexes());
 }
@@ -172,40 +179,49 @@ TEST(MemberConfig, ParseVotes) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 1.0),
+                                       << "votes"
+                                       << 1.0),
                             &tagConfig));
     ASSERT_TRUE(mc.isVoter());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 0 << "priority" << 0),
+                                       << "votes"
+                                       << 0
+                                       << "priority"
+                                       << 0),
                             &tagConfig));
     ASSERT_FALSE(mc.isVoter());
 
     // For backwards compatibility, truncate 1.X to 1, and 0.X to 0 (and -0.X to 0).
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 1.5),
+                                       << "votes"
+                                       << 1.5),
                             &tagConfig));
     ASSERT_TRUE(mc.isVoter());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 0.5),
+                                       << "votes"
+                                       << 0.5),
                             &tagConfig));
     ASSERT_FALSE(mc.isVoter());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << -0.5),
+                                       << "votes"
+                                       << -0.5),
                             &tagConfig));
     ASSERT_FALSE(mc.isVoter());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 2),
+                                       << "votes"
+                                       << 2),
                             &tagConfig));
 
     ASSERT_EQUALS(ErrorCodes::TypeMismatch,
                   mc.initialize(BSON("_id" << 0 << "host"
                                            << "h"
-                                           << "votes" << Date_t::fromMillisSinceEpoch(2)),
+                                           << "votes"
+                                           << Date_t::fromMillisSinceEpoch(2)),
                                 &tagConfig));
 }
 
@@ -214,24 +230,28 @@ TEST(MemberConfig, ParsePriority) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1),
+                                       << "priority"
+                                       << 1),
                             &tagConfig));
     ASSERT_EQUALS(1.0, mc.getPriority());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 0),
+                                       << "priority"
+                                       << 0),
                             &tagConfig));
     ASSERT_EQUALS(0.0, mc.getPriority());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 100.8),
+                                       << "priority"
+                                       << 100.8),
                             &tagConfig));
     ASSERT_EQUALS(100.8, mc.getPriority());
 
     ASSERT_EQUALS(ErrorCodes::TypeMismatch,
                   mc.initialize(BSON("_id" << 0 << "host"
                                            << "h"
-                                           << "priority" << Date_t::fromMillisSinceEpoch(2)),
+                                           << "priority"
+                                           << Date_t::fromMillisSinceEpoch(2)),
                                 &tagConfig));
 }
 
@@ -240,7 +260,8 @@ TEST(MemberConfig, ParseSlaveDelay) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "slaveDelay" << 100),
+                                       << "slaveDelay"
+                                       << 100),
                             &tagConfig));
     ASSERT_EQUALS(Seconds(100), mc.getSlaveDelay());
 }
@@ -250,10 +271,11 @@ TEST(MemberConfig, ParseTags) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "tags" << BSON("k1"
-                                                         << "v1"
-                                                         << "k2"
-                                                         << "v2")),
+                                       << "tags"
+                                       << BSON("k1"
+                                               << "v1"
+                                               << "k2"
+                                               << "v2")),
                             &tagConfig));
     ASSERT_EQUALS(5U, mc.getNumTags());
     ASSERT_EQUALS(5, std::distance(mc.tagsBegin(), mc.tagsEnd()));
@@ -284,14 +306,18 @@ TEST(MemberConfig, ValidateVotes) {
 
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 1.0),
+                                       << "votes"
+                                       << 1.0),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_TRUE(mc.isVoter());
 
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 0 << "priority" << 0),
+                                       << "votes"
+                                       << 0
+                                       << "priority"
+                                       << 0),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_FALSE(mc.isVoter());
@@ -299,21 +325,28 @@ TEST(MemberConfig, ValidateVotes) {
     // For backwards compatibility, truncate 1.X to 1, and 0.X to 0 (and -0.X to 0).
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 1.5),
+                                       << "votes"
+                                       << 1.5),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_TRUE(mc.isVoter());
 
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 0.5 << "priority" << 0),
+                                       << "votes"
+                                       << 0.5
+                                       << "priority"
+                                       << 0),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_FALSE(mc.isVoter());
 
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << -0.5 << "priority" << 0),
+                                       << "votes"
+                                       << -0.5
+                                       << "priority"
+                                       << 0),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_FALSE(mc.isVoter());
@@ -321,13 +354,15 @@ TEST(MemberConfig, ValidateVotes) {
     // Invalid values
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 2),
+                                       << "votes"
+                                       << 2),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
 
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << -1),
+                                       << "votes"
+                                       << -1),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
 }
@@ -337,22 +372,26 @@ TEST(MemberConfig, ValidatePriorityRanges) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 0),
+                                       << "priority"
+                                       << 0),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1000),
+                                       << "priority"
+                                       << 1000),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << -1),
+                                       << "priority"
+                                       << -1),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1001),
+                                       << "priority"
+                                       << 1001),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
 }
@@ -362,22 +401,34 @@ TEST(MemberConfig, ValidateSlaveDelays) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 0 << "slaveDelay" << 0),
+                                       << "priority"
+                                       << 0
+                                       << "slaveDelay"
+                                       << 0),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 0 << "slaveDelay" << 3600 * 10),
+                                       << "priority"
+                                       << 0
+                                       << "slaveDelay"
+                                       << 3600 * 10),
                             &tagConfig));
     ASSERT_OK(mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 0 << "slaveDelay" << -1),
+                                       << "priority"
+                                       << 0
+                                       << "slaveDelay"
+                                       << -1),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 0 << "slaveDelay" << 3600 * 24 * 400),
+                                       << "priority"
+                                       << 0
+                                       << "slaveDelay"
+                                       << 3600 * 24 * 400),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
 }
@@ -387,7 +438,10 @@ TEST(MemberConfig, ValidatePriorityAndSlaveDelayRelationship) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1 << "slaveDelay" << 60),
+                                       << "priority"
+                                       << 1
+                                       << "slaveDelay"
+                                       << 60),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
 }
@@ -397,12 +451,18 @@ TEST(MemberConfig, ValidatePriorityAndHiddenRelationship) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1 << "hidden" << true),
+                                       << "priority"
+                                       << 1
+                                       << "hidden"
+                                       << true),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1 << "hidden" << false),
+                                       << "priority"
+                                       << 1
+                                       << "hidden"
+                                       << false),
                             &tagConfig));
     ASSERT_OK(mc.validate());
 }
@@ -412,13 +472,19 @@ TEST(MemberConfig, ValidatePriorityAndBuildIndexesRelationship) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1 << "buildIndexes" << false),
+                                       << "priority"
+                                       << 1
+                                       << "buildIndexes"
+                                       << false),
                             &tagConfig));
 
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "priority" << 1 << "buildIndexes" << true),
+                                       << "priority"
+                                       << 1
+                                       << "buildIndexes"
+                                       << true),
                             &tagConfig));
     ASSERT_OK(mc.validate());
 }
@@ -428,25 +494,38 @@ TEST(MemberConfig, ValidateArbiterVotesRelationship) {
     MemberConfig mc;
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 1 << "arbiterOnly" << true),
-                            &tagConfig));
-    ASSERT_OK(mc.validate());
-
-    ASSERT_OK(
-        mc.initialize(BSON("_id" << 0 << "host"
-                                 << "h"
-                                 << "votes" << 0 << "priority" << 0 << "arbiterOnly" << false),
-                      &tagConfig));
-    ASSERT_OK(mc.validate());
-    ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
-                                       << "h"
-                                       << "votes" << 1 << "arbiterOnly" << false),
+                                       << "votes"
+                                       << 1
+                                       << "arbiterOnly"
+                                       << true),
                             &tagConfig));
     ASSERT_OK(mc.validate());
 
     ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
                                        << "h"
-                                       << "votes" << 0 << "arbiterOnly" << true),
+                                       << "votes"
+                                       << 0
+                                       << "priority"
+                                       << 0
+                                       << "arbiterOnly"
+                                       << false),
+                            &tagConfig));
+    ASSERT_OK(mc.validate());
+    ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
+                                       << "h"
+                                       << "votes"
+                                       << 1
+                                       << "arbiterOnly"
+                                       << false),
+                            &tagConfig));
+    ASSERT_OK(mc.validate());
+
+    ASSERT_OK(mc.initialize(BSON("_id" << 0 << "host"
+                                       << "h"
+                                       << "votes"
+                                       << 0
+                                       << "arbiterOnly"
+                                       << true),
                             &tagConfig));
     ASSERT_EQUALS(ErrorCodes::BadValue, mc.validate());
 }
