@@ -32,14 +32,15 @@
 
 #include "mongo/db/storage/storage_engine_metadata.h"
 
-#include <cstdio>
 #include <boost/filesystem.hpp>
 #include <boost/optional.hpp>
+#include <cstdio>
 #include <fstream>
 #include <limits>
 #include <ostream>
 #include <vector>
 
+#include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/log.h"
@@ -47,6 +48,8 @@
 
 namespace mongo {
 
+namespace dps = ::mongo::dotted_path_support;
+
 namespace {
 
 const std::string kMetadataBasename = "storage.bson";
@@ -141,24 +144,27 @@ Status StorageEngineMetadata::read() {
     }
 
     std::vector<char> buffer(fileSize);
-    std::string filename = metadataPath.string();
     try {
-        std::ifstream ifs(filename.c_str(), std::ios_base::in | std::ios_base::binary);
+        std::ifstream ifs(metadataPath.c_str(), std::ios_base::in | std::ios_base::binary);
         if (!ifs) {
             return Status(ErrorCodes::FileNotOpen,
-                          str::stream() << "Failed to read metadata from " << filename);
+                          str::stream() << "Failed to read metadata from "
+                                        << metadataPath.string());
         }
 
         // Read BSON from file
         ifs.read(&buffer[0], buffer.size());
         if (!ifs) {
             return Status(ErrorCodes::FileStreamFailed,
-                          str::stream() << "Unable to read BSON data from " << filename);
+                          str::stream() << "Unable to read BSON data from "
+                                        << metadataPath.string());
         }
     } catch (const std::exception& ex) {
         return Status(ErrorCodes::FileStreamFailed,
-                      str::stream() << "Unexpected error reading BSON data from " << filename
-                                    << ": " << ex.what());
+                      str::stream() << "Unexpected error reading BSON data from "
+                                    << metadataPath.string()
+                                    << ": "
+                                    << ex.what());
     }
 
     BSONObj obj;
@@ -166,12 +172,13 @@ Status StorageEngineMetadata::read() {
         obj = BSONObj(&buffer[0]);
     } catch (DBException& ex) {
         return Status(ErrorCodes::FailedToParse,
-                      str::stream() << "Failed to convert data in " << filename
-                                    << " to BSON: " << ex.what());
+                      str::stream() << "Failed to convert data in " << metadataPath.string()
+                                    << " to BSON: "
+                                    << ex.what());
     }
 
     // Validate 'storage.engine' field.
-    BSONElement storageEngineElement = obj.getFieldDotted("storage.engine");
+    BSONElement storageEngineElement = dps::extractElementAtPath(obj, "storage.engine");
     if (storageEngineElement.type() != mongo::String) {
         return Status(ErrorCodes::FailedToParse,
                       str::stream() << "The 'storage.engine' field in metadata must be a string: "
@@ -187,7 +194,7 @@ Status StorageEngineMetadata::read() {
     _storageEngine = storageEngine;
 
     // Read storage engine options generated by storage engine factory from startup options.
-    BSONElement storageEngineOptionsElement = obj.getFieldDotted("storage.options");
+    BSONElement storageEngineOptionsElement = dps::extractElementAtPath(obj, "storage.options");
     if (!storageEngineOptionsElement.eoo()) {
         if (!storageEngineOptionsElement.isABSONObj()) {
             return Status(ErrorCodes::FailedToParse,
@@ -210,12 +217,12 @@ Status StorageEngineMetadata::write() const {
     boost::filesystem::path metadataTempPath =
         boost::filesystem::path(_dbpath) / (kMetadataBasename + ".tmp");
     {
-        std::string filenameTemp = metadataTempPath.string();
-        std::ofstream ofs(filenameTemp.c_str(), std::ios_base::out | std::ios_base::binary);
+        std::ofstream ofs(metadataTempPath.c_str(), std::ios_base::out | std::ios_base::binary);
         if (!ofs) {
-            return Status(ErrorCodes::FileNotOpen,
-                          str::stream() << "Failed to write metadata to " << filenameTemp << ": "
-                                        << errnoWithDescription());
+            return Status(
+                ErrorCodes::FileNotOpen,
+                str::stream() << "Failed to write metadata to " << metadataTempPath.string() << ": "
+                              << errnoWithDescription());
         }
 
         BSONObj obj = BSON(
@@ -223,7 +230,9 @@ Status StorageEngineMetadata::write() const {
         ofs.write(obj.objdata(), obj.objsize());
         if (!ofs) {
             return Status(ErrorCodes::OperationFailed,
-                          str::stream() << "Failed to write BSON data to " << filenameTemp << ": "
+                          str::stream() << "Failed to write BSON data to "
+                                        << metadataTempPath.string()
+                                        << ": "
                                         << errnoWithDescription());
         }
     }
@@ -235,8 +244,11 @@ Status StorageEngineMetadata::write() const {
     } catch (const std::exception& ex) {
         return Status(ErrorCodes::FileRenameFailed,
                       str::stream() << "Unexpected error while renaming temporary metadata file "
-                                    << metadataTempPath.string() << " to " << metadataPath.string()
-                                    << ": " << ex.what());
+                                    << metadataTempPath.string()
+                                    << " to "
+                                    << metadataPath.string()
+                                    << ": "
+                                    << ex.what());
     }
 
     return Status::OK();
@@ -252,7 +264,9 @@ Status StorageEngineMetadata::validateStorageEngineOption<bool>(StringData field
     if (!element.isBoolean()) {
         return Status(ErrorCodes::FailedToParse,
                       str::stream() << "Expected boolean field " << fieldName << " but got "
-                                    << typeName(element.type()) << " instead: " << element);
+                                    << typeName(element.type())
+                                    << " instead: "
+                                    << element);
     }
     if (element.boolean() == expectedValue) {
         return Status::OK();
@@ -260,9 +274,12 @@ Status StorageEngineMetadata::validateStorageEngineOption<bool>(StringData field
     return Status(
         ErrorCodes::InvalidOptions,
         str::stream() << "Requested option conflicts with current storage engine option for "
-                      << fieldName << "; you requested " << (expectedValue ? "true" : "false")
+                      << fieldName
+                      << "; you requested "
+                      << (expectedValue ? "true" : "false")
                       << " but the current server storage is already set to "
-                      << (element.boolean() ? "true" : "false") << " and cannot be changed");
+                      << (element.boolean() ? "true" : "false")
+                      << " and cannot be changed");
 }
 
 }  // namespace mongo
