@@ -37,9 +37,9 @@
 #include <limits>
 
 #include "mongo/base/parse_number.h"
+#include "mongo/db/client.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/json.h"
-#include "mongo/db/operation_context_impl.h"
 #include "mongo/dbtests/dbtests.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/scripting/engine.h"
@@ -60,7 +60,7 @@ class BuiltinTests {
 public:
     void run() {
         // Run any tests included with the scripting engine
-        globalScriptEngine->runTest();
+        getGlobalScriptEngine()->runTest();
     }
 };
 
@@ -68,7 +68,7 @@ class BasicScope {
 public:
     void run() {
         unique_ptr<Scope> s;
-        s.reset(globalScriptEngine->newScope());
+        s.reset(getGlobalScriptEngine()->newScope());
 
         s->setNumber("x", 5);
         ASSERT(5 == s->getNumber("x"));
@@ -92,7 +92,7 @@ public:
     void run() {
         /* Currently reset does not clear data in v8 or spidermonkey scopes.  See SECURITY-10
         unique_ptr<Scope> s;
-        s.reset( globalScriptEngine->newScope() );
+        s.reset( getGlobalScriptEngine()->newScope() );
 
         s->setBoolean( "x" , true );
         ASSERT( s->getBoolean( "x" ) );
@@ -108,7 +108,7 @@ public:
     void run() {
         // Test falsy javascript values
         unique_ptr<Scope> s;
-        s.reset(globalScriptEngine->newScope());
+        s.reset(getGlobalScriptEngine()->newScope());
 
         ASSERT(!s->getBoolean("notSet"));
 
@@ -130,7 +130,7 @@ public:
 class SimpleFunctions {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         s->invoke("x=5;", 0, 0);
         ASSERT(5 == s->getNumber("x"));
@@ -196,7 +196,7 @@ private:
 class ExecLogError {
 public:
     void run() {
-        unique_ptr<Scope> scope(globalScriptEngine->newScope());
+        unique_ptr<Scope> scope(getGlobalScriptEngine()->newScope());
 
         // No error is logged when reportError == false.
         ASSERT(!scope->exec("notAFunction()", "foo", false, false, false));
@@ -212,7 +212,7 @@ public:
         // Don't check if we're using SpiderMonkey. Our threading model breaks
         // this test
         // TODO: figure out a way to check for SpiderMonkey
-        auto ivs = globalScriptEngine->getInterpreterVersionString();
+        auto ivs = getGlobalScriptEngine()->getInterpreterVersionString();
         std::string prefix("MozJS");
         if (ivs.compare(0, prefix.length(), prefix) != 0) {
             ASSERT(_logger.logged());
@@ -227,7 +227,7 @@ private:
 class InvokeLogError {
 public:
     void run() {
-        unique_ptr<Scope> scope(globalScriptEngine->newScope());
+        unique_ptr<Scope> scope(getGlobalScriptEngine()->newScope());
 
         // No error is logged for a valid statement.
         ASSERT_EQUALS(0, scope->invoke("validStatement = true", 0, 0));
@@ -243,7 +243,7 @@ public:
         // Don't check if we're using SpiderMonkey. Our threading model breaks
         // this test
         // TODO: figure out a way to check for SpiderMonkey
-        auto ivs = globalScriptEngine->getInterpreterVersionString();
+        auto ivs = getGlobalScriptEngine()->getInterpreterVersionString();
         std::string prefix("MozJS");
         if (ivs.compare(0, prefix.length(), prefix) != 0) {
             ASSERT(_logger.logged());
@@ -257,7 +257,7 @@ private:
 class ObjectMapping {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObj o = BSON("x" << 17.0 << "y"
                              << "eliot"
@@ -314,7 +314,7 @@ public:
 class ObjectDecoding {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         s->invoke("z = { num : 1 };", 0, 0);
         BSONObj out = s->getObject("z");
@@ -337,7 +337,7 @@ class JSOIDTests {
 public:
     void run() {
 #ifdef MOZJS
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         s->localConnect("blah");
 
@@ -368,7 +368,7 @@ public:
 class SetImplicit {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObj o = BSON("foo"
                          << "bar");
@@ -390,13 +390,14 @@ public:
 class ObjectModReadonlyTests {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObj o = BSON("x" << 17 << "y"
                              << "eliot"
                              << "z"
                              << "sara"
-                             << "zz" << BSONObj());
+                             << "zz"
+                             << BSONObj());
         s->setObject("blah", o, true);
 
         BSONObj out;
@@ -427,7 +428,7 @@ public:
 class OtherJSTypes {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         {
             // date
@@ -524,7 +525,7 @@ public:
 class SpecialDBTypes {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObjBuilder b;
         b.appendTimestamp("a", 123456789);
@@ -558,7 +559,7 @@ public:
 class TypeConservation {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         //  --  A  --
 
@@ -656,7 +657,7 @@ public:
 class NumberLong {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
         BSONObjBuilder b;
         long long val = (long long)(0xbabadeadbeefbaddULL);
         b.append("a", val);
@@ -698,7 +699,7 @@ public:
         s->setObject("z", BSON("z" << (long long)(4)));
         ASSERT(s->exec("y = {y:z.z.top}", "foo", false, true, false));
         out = s->getObject("y");
-        ASSERT_EQUALS(Undefined, out.firstElement().type());
+        ASSERT_EQUALS(NumberDouble, out.firstElement().type());
 
         ASSERT(s->exec("x = {x:z.z.floatApprox}", "foo", false, true, false));
         out = s->getObject("x");
@@ -715,7 +716,7 @@ public:
 class NumberLong2 {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObj in;
         {
@@ -735,14 +736,14 @@ public:
 
         ASSERT(s->exec((string) "y = " + outString, "foo2", false, true, false));
         BSONObj out = s->getObject("y");
-        ASSERT_EQUALS(in, out);
+        ASSERT_BSONOBJ_EQ(in, out);
     }
 };
 
 class NumberLongUnderLimit {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObjBuilder b;
         // limit is 2^53
@@ -788,7 +789,12 @@ public:
 class NumberDecimal {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        // Set the featureCompatibilityVersion to 3.4 so that BSON validation always uses
+        // BSONVersion::kLatest.
+        serverGlobalParams.featureCompatibility.version.store(
+            ServerGlobalParams::FeatureCompatibility::Version::k34);
+
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
         BSONObjBuilder b;
         Decimal128 val = Decimal128("2.010");
         b.append("a", val);
@@ -817,7 +823,12 @@ public:
 class NumberDecimalGetFromScope {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        // Set the featureCompatibilityVersion to 3.4 so that BSON validation always uses
+        // BSONVersion::kLatest.
+        serverGlobalParams.featureCompatibility.version.store(
+            ServerGlobalParams::FeatureCompatibility::Version::k34);
+
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
         ASSERT(s->exec("a = 5;", "a", false, true, false));
         ASSERT_TRUE(Decimal128(5).isEqual(s->getNumberDecimal("a")));
     }
@@ -826,7 +837,12 @@ public:
 class NumberDecimalBigObject {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        // Set the featureCompatibilityVersion to 3.4 so that BSON validation always uses
+        // BSONVersion::kLatest.
+        serverGlobalParams.featureCompatibility.version.store(
+            ServerGlobalParams::FeatureCompatibility::Version::k34);
+
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         BSONObj in;
         {
@@ -846,14 +862,14 @@ public:
 
         ASSERT(s->exec((string) "y = " + outString, "foo2", false, true, false));
         BSONObj out = s->getObject("y");
-        ASSERT_EQUALS(in, out);
+        ASSERT_BSONOBJ_EQ(in, out);
     }
 };
 
 class MaxTimestamp {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         // Timestamp 't' component can exceed max for int32_t.
         BSONObj in;
@@ -882,7 +898,7 @@ public:
     }
 
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         for (int i = 5; i < 100; i += 10) {
             s->setObject("a", build(i), false);
@@ -900,7 +916,7 @@ public:
 class ExecTimeout {
 public:
     void run() {
-        unique_ptr<Scope> scope(globalScriptEngine->newScope());
+        unique_ptr<Scope> scope(getGlobalScriptEngine()->newScope());
 
         // assert timeout occurred
         ASSERT(!scope->exec("var a = 1; while (true) { ; }", "ExecTimeout", false, true, false, 1));
@@ -913,7 +929,7 @@ public:
 class ExecNoTimeout {
 public:
     void run() {
-        unique_ptr<Scope> scope(globalScriptEngine->newScope());
+        unique_ptr<Scope> scope(getGlobalScriptEngine()->newScope());
 
         // assert no timeout occurred
         ASSERT(scope->exec("var a = function() { return 1; }",
@@ -931,7 +947,7 @@ public:
 class InvokeTimeout {
 public:
     void run() {
-        unique_ptr<Scope> scope(globalScriptEngine->newScope());
+        unique_ptr<Scope> scope(getGlobalScriptEngine()->newScope());
 
         // scope timeout after 500ms
         bool caught = false;
@@ -956,7 +972,7 @@ public:
 class InvokeNoTimeout {
 public:
     void run() {
-        unique_ptr<Scope> scope(globalScriptEngine->newScope());
+        unique_ptr<Scope> scope(getGlobalScriptEngine()->newScope());
 
         // invoke completes before timeout
         scope->invokeSafe(
@@ -979,14 +995,15 @@ public:
         reset();
     }
     void run() {
-        if (!globalScriptEngine->utf8Ok()) {
+        if (!getGlobalScriptEngine()->utf8Ok()) {
             mongo::unittest::log() << "warning: utf8 not supported" << endl;
             return;
         }
         string utf8ObjSpec = "{'_id':'\\u0001\\u007f\\u07ff\\uffff'}";
         BSONObj utf8Obj = fromjson(utf8ObjSpec);
 
-        OperationContextImpl txn;
+        const ServiceContext::UniqueOperationContext txnPtr = cc().makeOperationContext();
+        OperationContext& txn = *txnPtr;
         DBDirectClient client(&txn);
 
         client.insert(ns(), utf8Obj);
@@ -1006,7 +1023,8 @@ private:
     }
 
     void reset() {
-        OperationContextImpl txn;
+        const ServiceContext::UniqueOperationContext txnPtr = cc().makeOperationContext();
+        OperationContext& txn = *txnPtr;
         DBDirectClient client(&txn);
 
         client.dropCollection(ns());
@@ -1026,10 +1044,11 @@ public:
         reset();
     }
     void run() {
-        if (!globalScriptEngine->utf8Ok())
+        if (!getGlobalScriptEngine()->utf8Ok())
             return;
 
-        OperationContextImpl txn;
+        const ServiceContext::UniqueOperationContext txnPtr = cc().makeOperationContext();
+        OperationContext& txn = *txnPtr;
         DBDirectClient client(&txn);
 
         client.eval("unittest",
@@ -1038,7 +1057,8 @@ public:
 
 private:
     void reset() {
-        OperationContextImpl txn;
+        const ServiceContext::UniqueOperationContext txnPtr = cc().makeOperationContext();
+        OperationContext& txn = *txnPtr;
         DBDirectClient client(&txn);
 
         client.dropCollection(ns());
@@ -1052,11 +1072,11 @@ private:
 class InvalidUTF8Check {
 public:
     void run() {
-        if (!globalScriptEngine->utf8Ok())
+        if (!getGlobalScriptEngine()->utf8Ok())
             return;
 
         unique_ptr<Scope> s;
-        s.reset(globalScriptEngine->newScope());
+        s.reset(getGlobalScriptEngine()->newScope());
 
         BSONObj b;
         {
@@ -1082,7 +1102,7 @@ public:
 class CodeTests {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         {
             BSONObjBuilder b;
@@ -1116,8 +1136,14 @@ public:
     void run() {
         // Insert in Javascript -> Find using DBDirectClient
 
+        // Set the featureCompatibilityVersion to 3.4 so that BSON validation always uses
+        // BSONVersion::kLatest.
+        serverGlobalParams.featureCompatibility.version.store(
+            ServerGlobalParams::FeatureCompatibility::Version::k34);
+
         // Drop the collection
-        OperationContextImpl txn;
+        const ServiceContext::UniqueOperationContext txnPtr = cc().makeOperationContext();
+        OperationContext& txn = *txnPtr;
         DBDirectClient client(&txn);
 
         client.dropCollection("unittest.testroundtrip");
@@ -2086,7 +2112,7 @@ public:
     }
 
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         const char* foo = "asdas\0asdasd";
         const char* base64 = "YXNkYXMAYXNkYXNk";
@@ -2136,7 +2162,7 @@ public:
 class VarTests {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         ASSERT(s->exec("a = 5;", "a", false, true, false));
         ASSERT_EQUALS(5, s->getNumber("a"));
@@ -2153,7 +2179,7 @@ public:
         BSONObj empty;
 
         unique_ptr<Scope> s;
-        s.reset(globalScriptEngine->newScope());
+        s.reset(getGlobalScriptEngine()->newScope());
 
         ScriptingFunction f = s->createFunction("return this.x + 6;");
 
@@ -2171,13 +2197,13 @@ class ScopeOut {
 public:
     void run() {
         unique_ptr<Scope> s;
-        s.reset(globalScriptEngine->newScope());
+        s.reset(getGlobalScriptEngine()->newScope());
 
         s->invokeSafe("x = 5;", 0, 0);
         {
             BSONObjBuilder b;
             s->append(b, "z", "x");
-            ASSERT_EQUALS(BSON("z" << 5), b.obj());
+            ASSERT_BSONOBJ_EQ(BSON("z" << 5), b.obj());
         }
 
         s->invokeSafe("x = function(){ return 17; }", 0, 0);
@@ -2197,7 +2223,7 @@ class RenameTest {
 public:
     void run() {
         unique_ptr<Scope> s;
-        s.reset(globalScriptEngine->newScope());
+        s.reset(getGlobalScriptEngine()->newScope());
 
         s->setNumber("x", 5);
         ASSERT_EQUALS(5, s->getNumber("x"));
@@ -2225,11 +2251,12 @@ public:
         update.appendCode("value",
                           "function () { db.test.find().forEach(function(obj) { continue; }); }");
 
-        OperationContextImpl txn;
+        const ServiceContext::UniqueOperationContext txnPtr = cc().makeOperationContext();
+        OperationContext& txn = *txnPtr;
         DBDirectClient client(&txn);
         client.update("test.system.js", query.obj(), update.obj(), true /* upsert */);
 
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
         client.eval("test", "invalidstoredjs1()");
 
         BSONObj info;
@@ -2250,7 +2277,7 @@ public:
         uint8_t bits[] = {
             16, 0, 0, 0, 0x01, 'a', '\0', 0x61, 0x79, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0,
         };
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         s->setObject("val", BSONObj(reinterpret_cast<char*>(bits)).getOwned());
 
@@ -2262,7 +2289,7 @@ public:
 class NoReturnSpecified {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         s->invoke("x=5;", 0, 0);
         ASSERT_EQUALS(5, s->getNumber("__returnValue"));
@@ -2316,7 +2343,7 @@ public:
     }
 
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         s->injectNative("foo", callback, s.get());
         s->invoke("var x = 1; foo();", 0, 0);
@@ -2327,7 +2354,7 @@ public:
 class ErrorCodeFromInvoke {
 public:
     void run() {
-        unique_ptr<Scope> s(globalScriptEngine->newScope());
+        unique_ptr<Scope> s(getGlobalScriptEngine()->newScope());
 
         {
             bool threwException = false;
@@ -2363,10 +2390,7 @@ public:
 
 class All : public Suite {
 public:
-    All() : Suite("js") {
-        // Initialize the Javascript interpreter
-        ScriptEngine::setup();
-    }
+    All() : Suite("js") {}
 
     void setupTests() {
         add<BuiltinTests>();
@@ -2392,11 +2416,9 @@ public:
         add<NumberLong>();
         add<NumberLong2>();
 
-        if (Decimal128::enabled) {
-            add<NumberDecimal>();
-            add<NumberDecimalGetFromScope>();
-            add<NumberDecimalBigObject>();
-        }
+        add<NumberDecimal>();
+        add<NumberDecimalGetFromScope>();
+        add<NumberDecimalBigObject>();
 
         add<MaxTimestamp>();
         add<RenameTest>();
@@ -2470,19 +2492,17 @@ public:
         add<RoundTripTests::NumberInt>();
         add<RoundTripTests::Number>();
 
-        if (Decimal128::enabled) {
-            add<RoundTripTests::NumberDecimal>();
-            add<RoundTripTests::NumberDecimalNegative>();
-            add<RoundTripTests::NumberDecimalMax>();
-            add<RoundTripTests::NumberDecimalMin>();
-            add<RoundTripTests::NumberDecimalPositiveZero>();
-            add<RoundTripTests::NumberDecimalNegativeZero>();
-            add<RoundTripTests::NumberDecimalPositiveNaN>();
-            add<RoundTripTests::NumberDecimalNegativeNaN>();
-            add<RoundTripTests::NumberDecimalPositiveInfinity>();
-            add<RoundTripTests::NumberDecimalNegativeInfinity>();
-            add<RoundTripTests::NumberDecimalPrecision>();
-        }
+        add<RoundTripTests::NumberDecimal>();
+        add<RoundTripTests::NumberDecimalNegative>();
+        add<RoundTripTests::NumberDecimalMax>();
+        add<RoundTripTests::NumberDecimalMin>();
+        add<RoundTripTests::NumberDecimalPositiveZero>();
+        add<RoundTripTests::NumberDecimalNegativeZero>();
+        add<RoundTripTests::NumberDecimalPositiveNaN>();
+        add<RoundTripTests::NumberDecimalNegativeNaN>();
+        add<RoundTripTests::NumberDecimalPositiveInfinity>();
+        add<RoundTripTests::NumberDecimalNegativeInfinity>();
+        add<RoundTripTests::NumberDecimalPrecision>();
 
         add<RoundTripTests::UUID>();
         add<RoundTripTests::HexData>();
