@@ -25,7 +25,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kASIO
 
 #include "mongo/platform/basic.h"
@@ -35,12 +34,12 @@
 #include <type_traits>
 #include <utility>
 
+#include "mongo/base/static_assert.h"
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/executor/async_stream_interface.h"
 #include "mongo/executor/async_stream_interface.h"
 #include "mongo/executor/connection_pool_asio.h"
-#include "mongo/executor/downconvert_find_and_getmore_commands.h"
 #include "mongo/rpc/factory.h"
 #include "mongo/rpc/metadata/metadata_hook.h"
 #include "mongo/rpc/protocol.h"
@@ -69,17 +68,18 @@ using ResponseStatus = TaskExecutor::ResponseStatus;
 
 // A type conforms to the NetworkHandler concept if it is a callable type that takes a
 // std::error_code and std::size_t and returns void. The std::error_code parameter is used
-// to inform the handler if the asynchronous operation it was waiting on succeeded, and the size_t
-// parameter conveys how many bytes were read or written.
+// to inform the handler if the asynchronous operation it was waiting on succeeded, and the
+// size_t parameter conveys how many bytes were read or written.
 template <typename FunctionLike>
 using IsNetworkHandler =
     std::is_convertible<FunctionLike, stdx::function<void(std::error_code, std::size_t)>>;
 
 template <typename Handler>
 void asyncSendMessage(AsyncStreamInterface& stream, Message* m, Handler&& handler) {
-    static_assert(IsNetworkHandler<Handler>::value,
-                  "Handler passed to asyncSendMessage does not conform to NetworkHandler concept");
-    m->header().setResponseTo(0);
+    MONGO_STATIC_ASSERT_MSG(
+        IsNetworkHandler<Handler>::value,
+        "Handler passed to asyncSendMessage does not conform to NetworkHandler concept");
+    m->header().setResponseToMsgId(0);
     m->header().setId(nextMessageId());
     // TODO: Some day we may need to support vector messages.
     fassert(28708, m->buf() != 0);
@@ -90,7 +90,7 @@ template <typename Handler>
 void asyncRecvMessageHeader(AsyncStreamInterface& stream,
                             MSGHEADER::Value* header,
                             Handler&& handler) {
-    static_assert(
+    MONGO_STATIC_ASSERT_MSG(
         IsNetworkHandler<Handler>::value,
         "Handler passed to asyncRecvMessageHeader does not conform to NetworkHandler concept");
     stream.read(asio::buffer(header->view().view2ptr(), sizeof(decltype(*header))),
@@ -102,7 +102,7 @@ void asyncRecvMessageBody(AsyncStreamInterface& stream,
                           MSGHEADER::Value* header,
                           Message* m,
                           Handler&& handler) {
-    static_assert(
+    MONGO_STATIC_ASSERT_MSG(
         IsNetworkHandler<Handler>::value,
         "Handler passed to asyncRecvMessageBody does not conform to NetworkHandler concept");
     // validate message length
@@ -119,7 +119,7 @@ void asyncRecvMessageBody(AsyncStreamInterface& stream,
 
     int z = (len + 1023) & 0xfffffc00;
     invariant(z >= len);
-    m->setData(reinterpret_cast<char*>(mongoMalloc(z)), true);
+    m->setData(SharedBuffer::allocate(z));
     MsgData::View mdView = m->buf();
 
     // copy header data into master buffer
@@ -143,13 +143,16 @@ ResponseStatus decodeRPC(Message* received,
         if (reply->getProtocol() != protocol) {
             auto requestProtocol = rpc::toString(static_cast<rpc::ProtocolSet>(protocol));
             if (!requestProtocol.isOK())
-                return requestProtocol.getStatus();
+                return {requestProtocol.getStatus(), elapsed};
 
-            return Status(ErrorCodes::RPCProtocolNegotiationFailed,
-                          str::stream() << "Mismatched RPC protocols - request was '"
-                                        << requestProtocol.getValue().toString() << "' '"
-                                        << " but reply was '"
-                                        << networkOpToString(received->operation()) << "'");
+            return {ErrorCodes::RPCProtocolNegotiationFailed,
+                    str::stream() << "Mismatched RPC protocols - request was '"
+                                  << requestProtocol.getValue().toString()
+                                  << "' '"
+                                  << " but reply was '"
+                                  << networkOpToString(received->operation())
+                                  << "'",
+                    elapsed};
         }
         auto commandReply = reply->getCommandReply();
         auto replyMetadata = reply->getMetadata();
@@ -159,26 +162,25 @@ ResponseStatus decodeRPC(Message* received,
             auto listenStatus = callNoexcept(
                 *metadataHook, &rpc::EgressMetadataHook::readReplyMetadata, source, replyMetadata);
             if (!listenStatus.isOK()) {
-                return listenStatus;
+                return {listenStatus, elapsed};
             }
         }
 
         return {RemoteCommandResponse(
             std::move(*received), std::move(commandReply), std::move(replyMetadata), elapsed)};
     } catch (...) {
-        return exceptionToStatus();
+        return {exceptionToStatus(), elapsed};
     }
 }
 
 }  // namespace
 
 NetworkInterfaceASIO::AsyncCommand::AsyncCommand(AsyncConnection* conn,
-                                                 CommandType type,
                                                  Message&& command,
                                                  Date_t now,
                                                  const HostAndPort& target)
-    : _conn(conn), _type(type), _toSend(std::move(command)), _start(now), _target(target) {
-    _toSend.header().setResponseTo(0);
+    : _conn(conn), _toSend(std::move(command)), _start(now), _target(target) {
+    _toSend.header().setResponseToMsgId(0);
 }
 
 NetworkInterfaceASIO::AsyncConnection& NetworkInterfaceASIO::AsyncCommand::conn() {
@@ -197,25 +199,27 @@ MSGHEADER::Value& NetworkInterfaceASIO::AsyncCommand::header() {
     return _header;
 }
 
-ResponseStatus NetworkInterfaceASIO::AsyncCommand::response(rpc::Protocol protocol,
+ResponseStatus NetworkInterfaceASIO::AsyncCommand::response(AsyncOp* op,
+                                                            rpc::Protocol protocol,
                                                             Date_t now,
                                                             rpc::EgressMetadataHook* metadataHook) {
     auto& received = _toRecv;
-    switch (_type) {
-        case CommandType::kRPC: {
-            return decodeRPC(&received, protocol, now - _start, _target, metadataHook);
-        }
-        case CommandType::kDownConvertedFind:
-        case CommandType::kDownConvertedGetMore: {
-            auto ns = DbMessage(_toSend).getns();
-            return prepareOpReplyErrorResponse(_toSend.header().getId(), ns, &received);
+    if (received.operation() == dbCompressed) {
+        auto swm = conn().getCompressorManager().decompressMessage(received);
+        if (!swm.isOK()) {
+            return swm.getStatus();
         }
+        received = std::move(swm.getValue());
     }
-    MONGO_UNREACHABLE;
+
+    auto rs = decodeRPC(&received, protocol, now - _start, _target, metadataHook);
+    if (rs.isOK())
+        op->setResponseMetadata(rs.metadata);
+    return rs;
 }
 
 void NetworkInterfaceASIO::_startCommand(AsyncOp* op) {
-    LOG(3) << "running command " << op->request().cmdObj << " against database "
+    LOG(3) << "running command " << redact(op->request().cmdObj) << " against database "
            << op->request().dbname << " across network to " << op->request().target.toString();
     if (inShutdown()) {
         return;
@@ -233,7 +237,6 @@ void NetworkInterfaceASIO::_beginCommunication(AsyncOp* op) {
     // return to the connection pool's get() callback with _inSetup == false,
     // so we can proceed with user operations after they return to this
     // codepath.
-
     if (op->_inSetup) {
         log() << "Successfully connected to " << op->request().target.toString();
         op->_inSetup = false;
@@ -241,69 +244,85 @@ void NetworkInterfaceASIO::_beginCommunication(AsyncOp* op) {
         return;
     }
 
-    LOG(3) << "Initiating asynchronous command: " << op->request().toString();
+    LOG(3) << "Initiating asynchronous command: " << redact(op->request().toString());
 
-    auto beginStatus = op->beginCommand(op->request(), _metadataHook.get());
+    auto beginStatus = op->beginCommand(op->request());
     if (!beginStatus.isOK()) {
         return _completeOperation(op, beginStatus);
     }
 
-    _asyncRunCommand(op,
-                     [this, op](std::error_code ec, size_t bytes) {
-                         _validateAndRun(op, ec, [this, op]() { _completedOpCallback(op); });
-                     });
+    _asyncRunCommand(op, [this, op](std::error_code ec, size_t bytes) {
+        _validateAndRun(op, ec, [this, op]() { _completedOpCallback(op); });
+    });
 }
 
 void NetworkInterfaceASIO::_completedOpCallback(AsyncOp* op) {
-    auto response = op->command()->response(op->operationProtocol(), now(), _metadataHook.get());
+    auto response =
+        op->command()->response(op, op->operationProtocol(), now(), _metadataHook.get());
     _completeOperation(op, response);
 }
 
 void NetworkInterfaceASIO::_networkErrorCallback(AsyncOp* op, const std::error_code& ec) {
-    if (ec.category() == mongoErrorCategory()) {
-        // If we get a Mongo error code, we can preserve it.
-        _completeOperation(op, Status(ErrorCodes::fromInt(ec.value()), ec.message()));
-    } else {
-        // If we get an asio or system error, we just convert it to a network error.
-        _completeOperation(op, Status(ErrorCodes::HostUnreachable, ec.message()));
-    }
+    ErrorCodes::Error errorCode = (ec.category() == mongoErrorCategory())
+        ? ErrorCodes::fromInt(ec.value())
+        : ErrorCodes::HostUnreachable;
+    _completeOperation(op, {errorCode, ec.message(), Milliseconds(now() - op->_start)});
 }
 
 // NOTE: This method may only be called by ASIO threads
 // (do not call from methods entered by TaskExecutor threads)
-void NetworkInterfaceASIO::_completeOperation(AsyncOp* op, const ResponseStatus& resp) {
+void NetworkInterfaceASIO::_completeOperation(AsyncOp* op, ResponseStatus resp) {
+    auto metadata = op->getResponseMetadata();
+    if (!metadata.isEmpty()) {
+        resp.metadata = metadata;
+    }
+
     // Cancel this operation's timeout. Note that the timeout callback may already be running,
     // may have run, or may have already been scheduled to run in the near future.
     if (op->_timeoutAlarm) {
         op->_timeoutAlarm->cancel();
     }
 
+    if (resp.status.code() == ErrorCodes::ExceededTimeLimit ||
+        resp.status.code() == ErrorCodes::NetworkInterfaceExceededTimeLimit) {
+        _numTimedOutOps.fetchAndAdd(1);
+    }
+
     if (op->_inSetup) {
         // If we are in setup we should only be here if we failed to connect.
-        invariant(!resp.isOK());
-        // If we fail during connection, we won't be able to access any of our members after calling
-        // op->finish().
-        log() << "Failed to connect to " << op->request().target << " - " << resp.getStatus();
+        MONGO_ASIO_INVARIANT(!resp.isOK(), "Failed to connect in setup", op);
+        // If we fail during connection, we won't be able to access any of op's members after
+        // calling finish(), so we return here.
+        log() << "Failed to connect to " << op->request().target << " - " << resp.status;
+        op->finish(std::move(resp));
+        return;
     }
 
     if (op->_inRefresh) {
         // If we are in refresh we should only be here if we failed to heartbeat.
-        invariant(!resp.isOK());
+        MONGO_ASIO_INVARIANT(!resp.isOK(), "In refresh, but did not fail to heartbeat", op);
         // If we fail during heartbeating, we won't be able to access any of op's members after
         // calling finish(), so we return here.
-        log() << "Failed to heartbeat to " << op->request().target << " - " << resp.getStatus();
-        op->finish(resp);
+        log() << "Failed asio heartbeat to " << op->request().target << " - "
+              << redact(resp.status);
+        _numFailedOps.fetchAndAdd(1);
+        op->finish(std::move(resp));
         return;
     }
 
     if (!resp.isOK()) {
-        // In the case that resp is not OK, but _inSetup is false, we are using a connection that
+        // In the case that resp is not OK, but _inSetup is false, we are using a connection
+        // that
         // we got from the pool to execute a command, but it failed for some reason.
-        LOG(2) << "Failed to execute command: " << op->request().toString()
-               << " reason: " << resp.getStatus();
-    }
+        LOG(2) << "Failed to execute command: " << redact(op->request().toString())
+               << " reason: " << redact(resp.status);
 
-    op->finish(resp);
+        if (resp.status.code() != ErrorCodes::CallbackCanceled) {
+            _numFailedOps.fetchAndAdd(1);
+        }
+    } else {
+        _numSucceededOps.fetchAndAdd(1);
+    }
 
     std::unique_ptr<AsyncOp> ownedOp;
 
@@ -312,24 +331,24 @@ void NetworkInterfaceASIO::_completeOperation(AsyncOp* op, const ResponseStatus&
 
         auto iter = _inProgress.find(op);
 
-        // This can happen if we fail during setup.
-        if (iter == _inProgress.end()) {
-            return;
-        }
+        MONGO_ASIO_INVARIANT_INLOCK(
+            iter != _inProgress.end(), "Could not find AsyncOp in _inProgress", op);
 
         ownedOp = std::move(iter->second);
         _inProgress.erase(iter);
     }
 
-    invariant(ownedOp);
+    op->finish(std::move(resp));
+
+    MONGO_ASIO_INVARIANT(static_cast<bool>(ownedOp), "Invalid AsyncOp", op);
 
     auto conn = std::move(op->_connectionPoolHandle);
     auto asioConn = static_cast<connection_pool_asio::ASIOConnection*>(conn.get());
 
-    // Prevent any other threads or callbacks from accessing this op so we may safely complete and
-    // destroy it. It is key that we do this after we remove the op from the _inProgress map or
-    // someone else in cancelCommand could read the bumped generation and cancel the next command
-    // that uses this op. See SERVER-20556.
+    // Prevent any other threads or callbacks from accessing this op so we may safely complete
+    // and destroy it. It is key that we do this after we remove the op from the _inProgress map
+    // or someone else in cancelCommand could read the bumped generation and cancel the next
+    // command that uses this op. See SERVER-20556.
     {
         stdx::lock_guard<stdx::mutex> lk(op->_access->mutex);
         ++(op->_access->id);
@@ -341,7 +360,7 @@ void NetworkInterfaceASIO::_completeOperation(AsyncOp* op, const ResponseStatus&
 
     asioConn->bindAsyncOp(std::move(ownedOp));
     if (!resp.isOK()) {
-        asioConn->indicateFailure(resp.getStatus());
+        asioConn->indicateFailure(resp.status);
     } else {
         asioConn->indicateUsed();
         asioConn->indicateSuccess();
@@ -375,41 +394,36 @@ void NetworkInterfaceASIO::_asyncRunCommand(AsyncOp* op, NetworkOpHandler handle
     // Step 3
     auto recvHeaderCallback = [this, cmd, handler, recvMessageCallback, op](std::error_code ec,
                                                                             size_t bytes) {
-        // The operation could have been canceled after starting the command, but before receiving
-        // the header
-        _validateAndRun(op,
-                        ec,
-                        [this, op, recvMessageCallback, ec, bytes, cmd, handler] {
-                            // validate response id
-                            uint32_t expectedId = cmd->toSend().header().getId();
-                            uint32_t actualId = cmd->header().constView().getResponseTo();
-                            if (actualId != expectedId) {
-                                LOG(3) << "got wrong response:"
-                                       << " expected response id: " << expectedId
-                                       << ", got response id: " << actualId;
-                                return handler(make_error_code(ErrorCodes::ProtocolError), bytes);
-                            }
+        // The operation could have been canceled after starting the command, but before
+        // receiving the header
+        _validateAndRun(op, ec, [this, op, recvMessageCallback, ec, bytes, cmd, handler] {
+            // validate response id
+            uint32_t expectedId = cmd->toSend().header().getId();
+            uint32_t actualId = cmd->header().constView().getResponseToMsgId();
+            if (actualId != expectedId) {
+                LOG(3) << "got wrong response:"
+                       << " expected response id: " << expectedId
+                       << ", got response id: " << actualId;
+                return handler(make_error_code(ErrorCodes::ProtocolError), bytes);
+            }
 
-                            asyncRecvMessageBody(cmd->conn().stream(),
-                                                 &cmd->header(),
-                                                 &cmd->toRecv(),
-                                                 std::move(recvMessageCallback));
-                        });
+            asyncRecvMessageBody(cmd->conn().stream(),
+                                 &cmd->header(),
+                                 &cmd->toRecv(),
+                                 std::move(recvMessageCallback));
+        });
     };
 
     // Step 2
-    auto sendMessageCallback =
-        [this, cmd, handler, recvHeaderCallback, op](std::error_code ec, size_t bytes) {
-            _validateAndRun(op,
-                            ec,
-                            [this, cmd, op, recvHeaderCallback] {
-                                asyncRecvMessageHeader(cmd->conn().stream(),
-                                                       &cmd->header(),
-                                                       std::move(recvHeaderCallback));
-                            });
+    auto sendMessageCallback = [this, cmd, handler, recvHeaderCallback, op](std::error_code ec,
+                                                                            size_t bytes) {
+        _validateAndRun(op, ec, [this, cmd, op, recvHeaderCallback] {
+            asyncRecvMessageHeader(
+                cmd->conn().stream(), &cmd->header(), std::move(recvHeaderCallback));
+        });
 
 
-        };
+    };
 
     // Step 1
     asyncSendMessage(cmd->conn().stream(), &cmd->toSend(), std::move(sendMessageCallback));
@@ -433,23 +447,21 @@ void NetworkInterfaceASIO::_runConnectionHook(AsyncOp* op) {
         return _beginCommunication(op);
     }
 
-    auto beginStatus = op->beginCommand(*optionalRequest, _metadataHook.get());
+    auto beginStatus = op->beginCommand(*optionalRequest);
     if (!beginStatus.isOK()) {
         return _completeOperation(op, beginStatus);
     }
 
     auto finishHook = [this, op]() {
         auto response =
-            op->command()->response(op->operationProtocol(), now(), _metadataHook.get());
+            op->command()->response(op, op->operationProtocol(), now(), _metadataHook.get());
 
         if (!response.isOK()) {
-            return _completeOperation(op, response.getStatus());
+            return _completeOperation(op, response);
         }
 
-        auto handleStatus = callNoexcept(*_hook,
-                                         &NetworkConnectionHook::handleReply,
-                                         op->request().target,
-                                         std::move(response.getValue()));
+        auto handleStatus = callNoexcept(
+            *_hook, &NetworkConnectionHook::handleReply, op->request().target, std::move(response));
 
         if (!handleStatus.isOK()) {
             return _completeOperation(op, handleStatus);
@@ -458,10 +470,9 @@ void NetworkInterfaceASIO::_runConnectionHook(AsyncOp* op) {
         return _beginCommunication(op);
     };
 
-    return _asyncRunCommand(op,
-                            [this, op, finishHook](std::error_code ec, std::size_t bytes) {
-                                _validateAndRun(op, ec, finishHook);
-                            });
+    return _asyncRunCommand(op, [this, op, finishHook](std::error_code ec, std::size_t bytes) {
+        _validateAndRun(op, ec, finishHook);
+    });
 }
 
 
