@@ -114,6 +114,11 @@ Status AsyncResultsMerger::setAwaitDataTimeout(Milliseconds awaitDataTimeout) {
     return Status::OK();
 }
 
+void AsyncResultsMerger::setOperationContext(OperationContext* txn) {
+    stdx::lock_guard<stdx::mutex> lk(_mutex);
+    _params.txn = txn;
+}
+
 bool AsyncResultsMerger::ready() {
     stdx::lock_guard<stdx::mutex> lk(_mutex);
     return ready_inlock();
@@ -177,7 +182,7 @@ bool AsyncResultsMerger::readyUnsorted_inlock() {
     return allExhausted;
 }
 
-StatusWith<boost::optional<BSONObj>> AsyncResultsMerger::nextReady() {
+StatusWith<ClusterQueryResult> AsyncResultsMerger::nextReady() {
     stdx::lock_guard<stdx::mutex> lk(_mutex);
     dassert(ready_inlock());
     if (_lifecycleState != kAlive) {
@@ -190,19 +195,19 @@ StatusWith<boost::optional<BSONObj>> AsyncResultsMerger::nextReady() {
 
     if (_eofNext) {
         _eofNext = false;
-        return {boost::none};
+        return {ClusterQueryResult()};
     }
 
     const bool hasSort = !_params.sort.isEmpty();
     return hasSort ? nextReadySorted() : nextReadyUnsorted();
 }
 
-boost::optional<BSONObj> AsyncResultsMerger::nextReadySorted() {
+ClusterQueryResult AsyncResultsMerger::nextReadySorted() {
     // Tailable cursors cannot have a sort.
     invariant(!_params.isTailable);
 
     if (_mergeQueue.empty()) {
-        return boost::none;
+        return {};
     }
 
     size_t smallestRemote = _mergeQueue.top();
@@ -211,7 +216,7 @@ boost::optional<BSONObj> AsyncResultsMerger::nextReadySorted() {
     invariant(!_remotes[smallestRemote].docBuffer.empty());
     invariant(_remotes[smallestRemote].status.isOK());
 
-    BSONObj front = _remotes[smallestRemote].docBuffer.front();
+    ClusterQueryResult front = _remotes[smallestRemote].docBuffer.front();
     _remotes[smallestRemote].docBuffer.pop();
 
     // Re-populate the merging queue with the next result from 'smallestRemote', if it has a
@@ -223,14 +228,14 @@ boost::optional<BSONObj> AsyncResultsMerger::nextReadySorted() {
     return front;
 }
 
-boost::optional<BSONObj> AsyncResultsMerger::nextReadyUnsorted() {
+ClusterQueryResult AsyncResultsMerger::nextReadyUnsorted() {
     size_t remotesAttempted = 0;
     while (remotesAttempted < _remotes.size()) {
         // It is illegal to call this method if there is an error received from any shard.
         invariant(_remotes[_gettingFromRemote].status.isOK());
 
         if (_remotes[_gettingFromRemote].hasNext()) {
-            BSONObj front = _remotes[_gettingFromRemote].docBuffer.front();
+            ClusterQueryResult front = _remotes[_gettingFromRemote].docBuffer.front();
             _remotes[_gettingFromRemote].docBuffer.pop();
 
             if (_params.isTailable && !_remotes[_gettingFromRemote].hasNext()) {
@@ -250,7 +255,7 @@ boost::optional<BSONObj> AsyncResultsMerger::nextReadyUnsorted() {
         }
     }
 
-    return boost::none;
+    return {};
 }
 
 Status AsyncResultsMerger::askForNextBatch_inlock(size_t remoteIndex) {
@@ -276,7 +281,8 @@ Status AsyncResultsMerger::askForNextBatch_inlock(size_t remoteIndex) {
                                 adjustedBatchSize,
                                 _awaitDataTimeout,
                                 boost::none,
-                                boost::none).toBSON();
+                                boost::none)
+                     .toBSON();
     } else {
         // Do the first time shard host resolution.
         invariant(_params.readPreference);
@@ -289,8 +295,11 @@ Status AsyncResultsMerger::askForNextBatch_inlock(size_t remoteIndex) {
         cmdObj = *remote.initialCmdObj;
     }
 
-    executor::RemoteCommandRequest request(
-        remote.getTargetHost(), _params.nsString.db().toString(), cmdObj, _metadataObj);
+    executor::RemoteCommandRequest request(remote.getTargetHost(),
+                                           _params.nsString.db().toString(),
+                                           cmdObj,
+                                           _metadataObj,
+                                           _params.txn);
 
     auto callbackStatus = _executor->scheduleRemoteCommand(
         request,
@@ -402,7 +411,7 @@ void AsyncResultsMerger::handleBatchResponse(
         // Make a best effort to parse the response and retrieve the cursor id. We need the cursor
         // id in order to issue a killCursors command against it.
         if (cbData.response.isOK()) {
-            auto cursorResponse = parseCursorResponse(cbData.response.getValue().data, remote);
+            auto cursorResponse = parseCursorResponse(cbData.response.data, remote);
             if (cursorResponse.isOK()) {
                 remote.cursorId = cursorResponse.getValue().getCursorId();
             }
@@ -431,49 +440,85 @@ void AsyncResultsMerger::handleBatchResponse(
     ScopeGuard signaller = MakeGuard(&AsyncResultsMerger::signalCurrentEventIfReady_inlock, this);
 
     StatusWith<CursorResponse> cursorResponseStatus(
-        cbData.response.isOK() ? parseCursorResponse(cbData.response.getValue().data, remote)
-                               : cbData.response.getStatus());
+        cbData.response.isOK() ? parseCursorResponse(cbData.response.data, remote)
+                               : cbData.response.status);
 
     if (!cursorResponseStatus.isOK()) {
-        // Notify the shard registry of the failure.
-        if (remote.shardId) {
-            auto shard = grid.shardRegistry()->getShardNoReload(*remote.shardId);
-            if (!shard) {
-                remote.status = Status(cursorResponseStatus.getStatus().code(),
-                                       str::stream() << "Could not find shard " << *remote.shardId
-                                                     << " containing host "
-                                                     << remote.getTargetHost().toString());
-            } else {
-                ShardRegistry::updateReplSetMonitor(
-                    shard->getTargeter(), remote.getTargetHost(), cursorResponseStatus.getStatus());
+        // In the case a read is performed against a view, the shard primary can return an error
+        // indicating that the underlying collection may be sharded. When this occurs the return
+        // message will include an expanded view definition and collection namespace which we need
+        // to store. This allows for a second attempt at the read directly against the underlying
+        // collection.
+        if (cursorResponseStatus.getStatus() ==
+            ErrorCodes::CommandOnShardedViewNotSupportedOnMongod) {
+            auto& responseObj = cbData.response.data;
+            if (!responseObj.hasField("resolvedView")) {
+                remote.status = Status(ErrorCodes::InternalError,
+                                       str::stream() << "Missing field 'resolvedView' in document: "
+                                                     << responseObj);
+                return;
             }
+
+            auto resolvedViewObj = responseObj.getObjectField("resolvedView");
+            if (resolvedViewObj.isEmpty()) {
+                remote.status = Status(ErrorCodes::InternalError,
+                                       str::stream() << "Field 'resolvedView' must be an object: "
+                                                     << responseObj);
+                return;
+            }
+
+            ClusterQueryResult result;
+            result.setViewDefinition(resolvedViewObj.getOwned());
+
+            remote.docBuffer.push(result);
+            remote.cursorId = 0;
+            remote.status = Status::OK();
+
+            if (!_params.sort.isEmpty()) {
+                // Push the index of this remote to the merge queue so that the resolved view is
+                // visible to nextReadySorted().
+                _mergeQueue.push(remoteIndex);
+            }
+            return;
         }
 
-        // If we can still retry the initial cursor establishment, reset the state so it can be
-        // retried the next time nextEvent is called. Never retry getMores to avoid
-        // accidentally skipping results.
-        if (!remote.cursorId && remote.retryCount < kMaxNumFailedHostRetryAttempts &&
-            ShardRegistry::kAllRetriableErrors.count(cursorResponseStatus.getStatus().code())) {
-            invariant(remote.docBuffer.empty());
-
-            LOG(1) << "Initial cursor establishment failed with retriable error and will be retried"
-                   << causedBy(cursorResponseStatus.getStatus());
-
-            ++remote.retryCount;
-
-            remote.status = Status::OK();  // Reset status so it can be retried.
-
-            // Signal the merger thread to make it retry this remote again.
-            if (_currentEvent.isValid()) {
-                // To prevent ourselves from signalling the event twice,
-                // we set '_currentEvent' as invalid after signalling it.
-                _executor->signalEvent(_currentEvent);
-                _currentEvent = executor::TaskExecutor::EventHandle();
-            }
-
-            return;
+        auto shard = remote.getShard();
+        if (!shard) {
+            remote.status = Status(cursorResponseStatus.getStatus().code(),
+                                   str::stream() << "Could not find shard " << *remote.shardId
+                                                 << " containing host "
+                                                 << remote.getTargetHost().toString());
         } else {
-            remote.status = cursorResponseStatus.getStatus();
+            shard->updateReplSetMonitor(remote.getTargetHost(), cursorResponseStatus.getStatus());
+
+            // If we can still retry the initial cursor establishment, reset the state so it can be
+            // retried the next time nextEvent is called. Never retry getMores to avoid
+            // accidentally skipping results.
+            if (!remote.cursorId && remote.retryCount < kMaxNumFailedHostRetryAttempts &&
+                shard->isRetriableError(cursorResponseStatus.getStatus().code(),
+                                        Shard::RetryPolicy::kIdempotent)) {
+                invariant(remote.shardId);
+                invariant(remote.docBuffer.empty());
+
+                LOG(1) << "Initial cursor establishment failed with retriable error and will be "
+                          "retried"
+                       << causedBy(redact(cursorResponseStatus.getStatus()));
+
+                ++remote.retryCount;
+                remote.status = Status::OK();  // Reset status so it can be retried.
+
+                // Signal the merger thread to make it retry this remote again.
+                if (_currentEvent.isValid()) {
+                    // To prevent ourselves from signalling the event twice,
+                    // we set '_currentEvent' as invalid after signalling it.
+                    _executor->signalEvent(_currentEvent);
+                    _currentEvent = executor::TaskExecutor::EventHandle();
+                }
+
+                return;
+            } else {
+                remote.status = cursorResponseStatus.getStatus();
+            }
         }
 
         // Unreachable host errors are swallowed if the 'allowPartialResults' option is set. We
@@ -482,7 +527,7 @@ void AsyncResultsMerger::handleBatchResponse(
             remote.status = Status::OK();
 
             // Clear the results buffer and cursor id.
-            std::queue<BSONObj> emptyBuffer;
+            std::queue<ClusterQueryResult> emptyBuffer;
             std::swap(remote.docBuffer, emptyBuffer);
             remote.cursorId = 0;
         }
@@ -502,11 +547,13 @@ void AsyncResultsMerger::handleBatchResponse(
             remote.status = Status(ErrorCodes::InternalError,
                                    str::stream() << "Missing field '"
                                                  << ClusterClientCursorParams::kSortKeyField
-                                                 << "' in document: " << obj);
+                                                 << "' in document: "
+                                                 << obj);
             return;
         }
 
-        remote.docBuffer.push(obj);
+        ClusterQueryResult result(obj);
+        remote.docBuffer.push(result);
         ++remote.fetchedCount;
     }
 
@@ -570,7 +617,7 @@ void AsyncResultsMerger::scheduleKillCursors_inlock() {
             BSONObj cmdObj = KillCursorsRequest(_params.nsString, {*remote.cursorId}).toBSON();
 
             executor::RemoteCommandRequest request(
-                remote.getTargetHost(), _params.nsString.db().toString(), cmdObj);
+                remote.getTargetHost(), _params.nsString.db().toString(), cmdObj, _params.txn);
 
             _executor->scheduleRemoteCommand(
                 request,
@@ -596,7 +643,7 @@ executor::TaskExecutor::EventHandle AsyncResultsMerger::kill() {
     // Make '_killCursorsScheduledEvent', which we will signal as soon as we have scheduled a
     // killCursors command to run on all the remote shards.
     auto statusWithEvent = _executor->makeEvent();
-    if (statusWithEvent.getStatus().code() == ErrorCodes::ShutdownInProgress) {
+    if (ErrorCodes::isShutdownError(statusWithEvent.getStatus().code())) {
         // The underlying task executor is shutting down.
         if (!haveOutstandingBatchRequests_inlock()) {
             _lifecycleState = kKillComplete;
@@ -647,15 +694,14 @@ Status AsyncResultsMerger::RemoteCursorData::resolveShardIdToHostAndPort(
     invariant(shardId);
     invariant(!cursorId);
 
-    const auto shard = grid.shardRegistry()->getShardNoReload(*shardId);
+    const auto shard = getShard();
     if (!shard) {
         return Status(ErrorCodes::ShardNotFound,
                       str::stream() << "Could not find shard " << *shardId);
     }
 
     // TODO: Pass down an OperationContext* to use here.
-    auto findHostStatus = shard->getTargeter()->findHost(
-        readPref, RemoteCommandTargeter::selectFindHostMaxWaitTime(nullptr));
+    auto findHostStatus = shard->getTargeter()->findHostWithMaxWait(readPref, Seconds{20});
     if (!findHostStatus.isOK()) {
         return findHostStatus.getStatus();
     }
@@ -665,17 +711,28 @@ Status AsyncResultsMerger::RemoteCursorData::resolveShardIdToHostAndPort(
     return Status::OK();
 }
 
+std::shared_ptr<Shard> AsyncResultsMerger::RemoteCursorData::getShard() {
+    invariant(shardId || _shardHostAndPort);
+    if (shardId) {
+        return grid.shardRegistry()->getShardNoReload(*shardId);
+    } else {
+        return grid.shardRegistry()->getShardNoReload(_shardHostAndPort->toString());
+    }
+}
+
 //
 // AsyncResultsMerger::MergingComparator
 //
 
 bool AsyncResultsMerger::MergingComparator::operator()(const size_t& lhs, const size_t& rhs) {
-    const BSONObj& leftDoc = _remotes[lhs].docBuffer.front();
-    const BSONObj& rightDoc = _remotes[rhs].docBuffer.front();
+    const ClusterQueryResult& leftDoc = _remotes[lhs].docBuffer.front();
+    const ClusterQueryResult& rightDoc = _remotes[rhs].docBuffer.front();
 
-    BSONObj leftDocKey = leftDoc[ClusterClientCursorParams::kSortKeyField].Obj();
-    BSONObj rightDocKey = rightDoc[ClusterClientCursorParams::kSortKeyField].Obj();
+    BSONObj leftDocKey = (*leftDoc.getResult())[ClusterClientCursorParams::kSortKeyField].Obj();
+    BSONObj rightDocKey = (*rightDoc.getResult())[ClusterClientCursorParams::kSortKeyField].Obj();
 
+    // This does not need to sort with a collator, since mongod has already mapped strings to their
+    // ICU comparison keys as part of the $sortKey meta projection.
     return leftDocKey.woCompare(rightDocKey, _sort, false /*considerFieldName*/) > 0;
 }
 
