@@ -32,6 +32,7 @@
 
 #include "mongo/scripting/mozjs/implscope.h"
 
+#include <js/CharacterEncoding.h>
 #include <jscustomallocator.h>
 #include <jsfriendapi.h>
 
@@ -43,6 +44,7 @@
 #include "mongo/scripting/mozjs/objectwrapper.h"
 #include "mongo/scripting/mozjs/valuereader.h"
 #include "mongo/scripting/mozjs/valuewriter.h"
+#include "mongo/stdx/memory.h"
 #include "mongo/stdx/mutex.h"
 #include "mongo/util/concurrency/threadlocal.h"
 #include "mongo/util/log.h"
@@ -66,11 +68,10 @@ const char* const MozJSImplScope::kInvokeResult = "__returnValue";
 namespace {
 
 /**
- * The maximum amount of memory to be given out per thread to mozilla. We
- * manage this by trapping all calls to malloc, free, etc. and keeping track of
- * counts in some thread locals
+ * The threshold (as a fraction of the max) after which garbage collection will be run during
+ * interrupts.
  */
-const size_t kMallocMemoryLimit = 1024ul * 1024 * 1024 * 1.1;
+const double kInterruptGCThreshold = 0.8;
 
 /**
  * The number of bytes to allocate after which garbage collection is run
@@ -90,6 +91,9 @@ const int kStackChunkSize = 8192;
 stdx::mutex gRuntimeCreationMutex;
 bool gFirstRuntimeCreated = false;
 
+bool closeToMaxMemory() {
+    return mongo::sm::get_total_bytes() > (kInterruptGCThreshold * mongo::sm::get_max_bytes());
+}
 }  // namespace
 
 MONGO_TRIVIALLY_CONSTRUCTIBLE_THREAD_LOCAL MozJSImplScope* kCurrentScope;
@@ -113,29 +117,39 @@ void MozJSImplScope::_reportError(JSContext* cx, const char* message, JSErrorRep
     auto scope = getScope(cx);
 
     if (!JSREPORT_IS_WARNING(report->flags)) {
-        str::stream ss;
-        ss << message;
 
-        // TODO: something far more elaborate that mimics the stack printing from v8
-        JS::RootedValue excn(cx);
-        if (JS_GetPendingException(cx, &excn) && excn.isObject()) {
-            JS::RootedValue stack(cx);
+        std::string exceptionMsg;
 
-            ObjectWrapper(cx, excn).getValue("stack", &stack);
+        try {
+            str::stream ss;
+            ss << message;
 
-            auto str = ValueWriter(cx, stack).toString();
+            // TODO: something far more elaborate that mimics the stack printing from v8
+            JS::RootedValue excn(cx);
+            if (JS_GetPendingException(cx, &excn) && excn.isObject()) {
+                JS::RootedValue stack(cx);
 
-            if (str.empty()) {
-                ss << " @" << report->filename << ":" << report->lineno << ":" << report->column
-                   << "\n";
-            } else {
-                ss << " :\n" << str;
+                ObjectWrapper(cx, excn).getValue("stack", &stack);
+
+                auto str = ValueWriter(cx, stack).toString();
+
+                if (str.empty()) {
+                    ss << " @" << report->filename << ":" << report->lineno << ":" << report->column
+                       << "\n";
+                } else {
+                    ss << " :\n" << str;
+                }
             }
+
+            exceptionMsg = ss;
+        } catch (const DBException& dbe) {
+            exceptionMsg = "Unknown error occured while processing exception";
+            log() << exceptionMsg << ":" << dbe.toString() << ":" << message;
         }
 
         scope->_status = Status(
             JSErrorReportToStatus(cx, report, ErrorCodes::JSInterpreterFailure, message).code(),
-            ss);
+            exceptionMsg);
     }
 }
 
@@ -169,6 +183,10 @@ void MozJSImplScope::kill() {
     JS_RequestInterruptCallback(_runtime);
 }
 
+void MozJSImplScope::interrupt() {
+    JS_RequestInterruptCallback(_runtime);
+}
+
 bool MozJSImplScope::isKillPending() const {
     return _pendingKill.load();
 }
@@ -187,7 +205,7 @@ bool MozJSImplScope::_interruptCallback(JSContext* cx) {
     JS_SetInterruptCallback(scope->_runtime, nullptr);
     auto guard = MakeGuard([&]() { JS_SetInterruptCallback(scope->_runtime, _interruptCallback); });
 
-    if (scope->_pendingGC.load()) {
+    if (scope->_pendingGC.load() || closeToMaxMemory()) {
         scope->_pendingGC.store(false);
         JS_GC(scope->_runtime);
     } else {
@@ -201,7 +219,8 @@ bool MozJSImplScope::_interruptCallback(JSContext* cx) {
     }
     // If we are on the right thread, in the middle of an operation, and we have a registered opCtx,
     // then we should check the opCtx for interrupts.
-    if ((scope->_mr._thread == PR_GetCurrentThread()) && (scope->_inOp > 0) && scope->_opCtx) {
+    if ((scope->_mr._thread.get() == PR_GetCurrentThread()) && (scope->_inOp > 0) &&
+        scope->_opCtx) {
         auto status = scope->_opCtx->checkForInterruptNoAssert();
         if (!status.isOK()) {
             scope->_status = status;
@@ -223,12 +242,22 @@ void MozJSImplScope::_gcCallback(JSRuntime* rt, JSGCStatus status, void* data) {
     }
 
     log() << "MozJS GC " << (status == JSGC_BEGIN ? "prologue" : "epilogue") << " heap stats - "
-          << " total: " << mongo::sm::get_total_bytes() << " limit: " << mongo::sm::get_max_bytes()
-          << std::endl;
+          << " total: " << mongo::sm::get_total_bytes() << " limit: " << mongo::sm::get_max_bytes();
 }
 
 MozJSImplScope::MozRuntime::MozRuntime(const MozJSScriptEngine* engine) {
-    mongo::sm::reset(kMallocMemoryLimit);
+    /**
+     * The maximum amount of memory to be given out per thread to mozilla. We
+     * manage this by trapping all calls to malloc, free, etc. and keeping track of
+     * counts in some thread locals
+     */
+
+    const auto jsHeapLimit = engine->getJSHeapLimitMB();
+    if (jsHeapLimit != 0 && jsHeapLimit < 10) {
+        warning() << "JavaScript may not be able to initialize with a heap limit less than 10MB.";
+    }
+    size_t mallocMemoryLimit = 1024ul * 1024 * jsHeapLimit;
+    mongo::sm::reset(mallocMemoryLimit);
 
     // If this runtime isn't running on an NSPR thread, then it is
     // running on a mongo thread. In that case, we need to insert a
@@ -236,7 +265,15 @@ MozJSImplScope::MozRuntime::MozRuntime(const MozJSScriptEngine* engine) {
     // without falling over.
     auto thread = PR_GetCurrentThread();
     if (!thread) {
-        PR_BindThread(_thread = PR_CreateFakeThread());
+        _thread = std::unique_ptr<PRThread, std::function<void(PRThread*)>>(
+            PR_CreateFakeThread(), [](PRThread* ptr) {
+                if (ptr) {
+                    invariant(PR_GetCurrentThread() == ptr);
+                    PR_DestroyFakeThread(ptr);
+                    PR_BindThread(nullptr);
+                }
+            });
+        PR_BindThread(_thread.get());
     }
 
     {
@@ -251,17 +288,19 @@ MozJSImplScope::MozRuntime::MozRuntime(const MozJSScriptEngine* engine) {
             gFirstRuntimeCreated = true;
         }
 
-        _runtime = JS_NewRuntime(kMaxBytesBeforeGC);
+        _runtime = std::unique_ptr<JSRuntime, std::function<void(JSRuntime*)>>(
+            JS_NewRuntime(kMaxBytesBeforeGC), [](JSRuntime* ptr) { JS_DestroyRuntime(ptr); });
         uassert(ErrorCodes::JSInterpreterFailure, "Failed to initialize JSRuntime", _runtime);
 
         // We turn on a variety of optimizations if the jit is enabled
         if (engine->isJITEnabled()) {
-            JS::RuntimeOptionsRef(_runtime)
+            JS::RuntimeOptionsRef(_runtime.get())
                 .setAsmJS(true)
+                .setThrowOnAsmJSValidationFailure(true)
                 .setBaseline(true)
                 .setIon(true)
-                .setNativeRegExp(true)
-                .setUnboxedObjects(true);
+                .setAsyncStack(false)
+                .setNativeRegExp(true);
         }
 
         const StackLocator locator;
@@ -278,33 +317,39 @@ MozJSImplScope::MozRuntime::MozRuntime(const MozJSScriptEngine* engine) {
             //
             // TODO: What if we are running on a platform with very
             // large pages, like 4MB?
-            JS_SetNativeStackQuota(_runtime, available.get() - (64 * 1024));
+            const int available_stack_space = available.get();
+
+#if defined(__powerpc64__) && defined(MONGO_CONFIG_DEBUG_BUILD)
+            // From experimentation, we need a larger reservation of 96k since debug ppc64le code
+            // needs more stack space to process stack overflow. In debug builds, more variables are
+            // stored on the stack which increases the stack pressure. It does not affects non-debug
+            // builds.
+            const int reserve_stack_space = 96 * 1024;
+#else
+            const int reserve_stack_space = 64 * 1024;
+#endif
+
+            JS_SetNativeStackQuota(_runtime.get(), available_stack_space - reserve_stack_space);
         }
 
         // The memory limit is in megabytes
-        JS_SetGCParametersBasedOnAvailableMemory(_runtime, kMallocMemoryLimit / (1024 * 1024));
+        JS_SetGCParametersBasedOnAvailableMemory(_runtime.get(), engine->getJSHeapLimitMB());
     }
 
-    _context = JS_NewContext(_runtime, kStackChunkSize);
+    _context = std::unique_ptr<JSContext, std::function<void(JSContext*)>>(
+        JS_NewContext(_runtime.get(), kStackChunkSize),
+        [](JSContext* ptr) { JS_DestroyContext(ptr); });
     uassert(ErrorCodes::JSInterpreterFailure, "Failed to initialize JSContext", _context);
-}
-
-MozJSImplScope::MozRuntime::~MozRuntime() {
-    JS_DestroyContext(_context);
-    JS_DestroyRuntime(_runtime);
-
-    if (_thread) {
-        invariant(PR_GetCurrentThread() == _thread);
-        PR_DestroyFakeThread(_thread);
-        PR_BindThread(nullptr);
-    }
+    uassert(ErrorCodes::ExceededMemoryLimit,
+            "Out of memory while trying to initialize javascript scope",
+            mallocMemoryLimit == 0 || mongo::sm::get_total_bytes() < mallocMemoryLimit);
 }
 
 MozJSImplScope::MozJSImplScope(MozJSScriptEngine* engine)
     : _engine(engine),
       _mr(engine),
-      _runtime(_mr._runtime),
-      _context(_mr._context),
+      _runtime(_mr._runtime.get()),
+      _context(_mr._context.get()),
       _globalProto(_context),
       _global(_globalProto.getProto()),
       _funcs(),
@@ -323,12 +368,12 @@ MozJSImplScope::MozJSImplScope(MozJSScriptEngine* engine)
       _bsonProto(_context),
       _codeProto(_context),
       _countDownLatchProto(_context),
-      _cursorProto(_context),
       _cursorHandleProto(_context),
+      _cursorProto(_context),
       _dbCollectionProto(_context),
+      _dbProto(_context),
       _dbPointerProto(_context),
       _dbQueryProto(_context),
-      _dbProto(_context),
       _dbRefProto(_context),
       _errorProto(_context),
       _jsThreadProto(_context),
@@ -338,13 +383,14 @@ MozJSImplScope::MozJSImplScope(MozJSScriptEngine* engine)
       _mongoHelpersProto(_context),
       _mongoLocalProto(_context),
       _nativeFunctionProto(_context),
+      _numberDecimalProto(_context),
       _numberIntProto(_context),
       _numberLongProto(_context),
-      _numberDecimalProto(_context),
       _objectProto(_context),
       _oidProto(_context),
       _regExpProto(_context),
-      _timestampProto(_context) {
+      _timestampProto(_context),
+      _uriProto(_context) {
     kCurrentScope = this;
 
     // The default is quite low and doesn't seem to directly correlate with
@@ -370,13 +416,13 @@ MozJSImplScope::MozJSImplScope(MozJSScriptEngine* engine)
     execSetup(JSFiles::assert);
     execSetup(JSFiles::types);
 
-    // install process-specific utilities in the global scope (dependancy: types.js, assert.js)
-    if (_engine->getScopeInitCallback())
-        _engine->getScopeInitCallback()(*this);
-
     // install global utility functions
     installGlobalUtils(*this);
     _mongoHelpersProto.install(_global);
+
+    // install process-specific utilities in the global scope (dependancy: types.js, assert.js)
+    if (_engine->getScopeInitCallback())
+        _engine->getScopeInitCallback()(*this);
 }
 
 MozJSImplScope::~MozJSImplScope() {
@@ -487,14 +533,20 @@ void MozJSImplScope::newFunction(StringData raw, JS::MutableHandleValue out) {
 
     JS::CompileOptions co(_context);
     setCompileOptions(&co);
-    _checkErrorState(JS::Evaluate(_context, _global, co, code.c_str(), code.length(), out));
+    _checkErrorState(JS::Evaluate(_context, co, code.c_str(), code.length(), out));
 }
 
 BSONObj MozJSImplScope::callThreadArgs(const BSONObj& args) {
     MozJSEntry entry(this);
 
     JS::RootedValue function(_context);
-    ValueReader(_context, &function).fromBSONElement(args.firstElement(), args, true);
+    auto firstElem = args.firstElement();
+
+    // The first argument must be the thread start function
+    if (firstElem.type() != mongo::Code)
+        uasserted(ErrorCodes::BadValue, "first thread argument must be a function");
+
+    getScope(_context)->newFunction(firstElem.valueStringData(), &function);
 
     int argc = args.nFields() - 1;
 
@@ -537,7 +589,7 @@ void MozJSImplScope::_MozJSCreateFunction(const char* raw,
     JS::CompileOptions co(_context);
     setCompileOptions(&co);
 
-    _checkErrorState(JS::Evaluate(_context, _global, co, code.c_str(), code.length(), fun));
+    _checkErrorState(JS::Evaluate(_context, co, code.c_str(), code.length(), fun));
     uassert(10232,
             "not a function",
             fun.isObject() && JS_ObjectIsFunction(_context, fun.toObjectOrNull()));
@@ -606,14 +658,16 @@ int MozJSImplScope::invoke(ScriptingFunction func,
 
     if (timeoutMs)
         _engine->getDeadlineMonitor().startDeadline(this, timeoutMs);
+    else {
+        _engine->getDeadlineMonitor().startDeadline(this, -1);
+    }
 
     JS::RootedValue out(_context);
     JS::RootedObject obj(_context, smrecv.toObjectOrNull());
 
     bool success = JS::Call(_context, obj, funcValue, args, &out);
 
-    if (timeoutMs)
-        _engine->getDeadlineMonitor().stopDeadline(this);
+    _engine->getDeadlineMonitor().stopDeadline(this);
 
     _checkErrorState(success);
 
@@ -643,23 +697,25 @@ bool MozJSImplScope::exec(StringData code,
 
     JS::CompileOptions co(_context);
     setCompileOptions(&co);
-    co.setFile(name.c_str());
+    co.setFileAndLine(name.c_str(), 1);
     JS::RootedScript script(_context);
 
-    bool success = JS::Compile(_context, _global, co, code.rawData(), code.size(), &script);
+    bool success = JS::Compile(_context, co, code.rawData(), code.size(), &script);
 
     if (_checkErrorState(success, reportError, assertOnError))
         return false;
 
-    if (timeoutMs)
+    if (timeoutMs) {
         _engine->getDeadlineMonitor().startDeadline(this, timeoutMs);
+    } else {
+        _engine->getDeadlineMonitor().startDeadline(this, -1);
+    }
 
     JS::RootedValue out(_context);
 
-    success = JS_ExecuteScript(_context, _global, script, &out);
+    success = JS_ExecuteScript(_context, script, &out);
 
-    if (timeoutMs)
-        _engine->getDeadlineMonitor().stopDeadline(this);
+    _engine->getDeadlineMonitor().stopDeadline(this);
 
     if (_checkErrorState(success, reportError, assertOnError))
         return false;
@@ -733,8 +789,6 @@ void MozJSImplScope::externalSetup() {
     if (_connectState == ConnectState::Local)
         uasserted(12512, "localConnect already called, can't call externalSetup");
 
-    mongo::sm::reset(0);
-
     // install db access functions in the global object
     installDBAccess();
 
@@ -766,13 +820,12 @@ void MozJSImplScope::installBSONTypes() {
     _nativeFunctionProto.install(_global);
     _numberIntProto.install(_global);
     _numberLongProto.install(_global);
-    if (Decimal128::enabled) {
-        _numberDecimalProto.install(_global);
-    }
+    _numberDecimalProto.install(_global);
     _objectProto.install(_global);
     _oidProto.install(_global);
     _regExpProto.install(_global);
     _timestampProto.install(_global);
+    _uriProto.install(_global);
 
     // This builtin map is a javascript 6 thing.  We want our version.  so
     // take theirs out
@@ -804,12 +857,18 @@ bool MozJSImplScope::_checkErrorState(bool success, bool reportError, bool asser
         if (JS_GetPendingException(_context, &excn) && excn.isObject()) {
             str::stream ss;
 
-            JS::RootedValue stack(_context);
+            auto stackStr = ObjectWrapper(_context, excn).getString(InternedString::stack);
+            auto fnameStr = ObjectWrapper(_context, excn).getString(InternedString::fileName);
+            auto lineNum = ObjectWrapper(_context, excn).getNumberInt(InternedString::lineNumber);
+            auto colNum = ObjectWrapper(_context, excn).getNumberInt(InternedString::columnNumber);
 
-            ObjectWrapper(_context, excn).getValue("stack", &stack);
-
-            ss << ValueWriter(_context, excn).toString() << " :\n"
-               << ValueWriter(_context, stack).toString();
+            if (fnameStr != "") {
+                ss << "[" << fnameStr << ":" << lineNum << ":" << colNum << "] ";
+            }
+            ss << ValueWriter(_context, excn).toString();
+            if (stackStr != "") {
+                ss << "\nStack trace:\n" << stackStr << "----------\n";
+            }
             _status = Status(ErrorCodes::JSInterpreterFailure, ss);
         } else {
             _status = Status(ErrorCodes::UnknownError, "Unknown Failure from JSInterpreter");
@@ -819,7 +878,7 @@ bool MozJSImplScope::_checkErrorState(bool success, bool reportError, bool asser
     _error = _status.reason();
 
     if (reportError)
-        error() << _error << std::endl;
+        error() << redact(_error);
 
     // Clear the status state
     auto status = std::move(_status);
