@@ -29,21 +29,28 @@
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kControl
 
-#include <malloc.h>
+#include "mongo/platform/basic.h"
+
+#include "processinfo.h"
+
 #include <iostream>
+#include <malloc.h>
+#include <sched.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <sys/mman.h>
 #include <sys/utsname.h>
+#include <unistd.h>
 #ifdef __UCLIBC__
 #include <features.h>
 #else
 #include <gnu/libc-version.h>
 #endif
 
-#include "processinfo.h"
-#include "boost/filesystem.hpp"
-#include <mongo/util/file.h>
+#include <boost/filesystem.hpp>
+#include <boost/none.hpp>
+#include <boost/optional.hpp>
+
+#include "mongo/util/file.h"
 #include "mongo/util/log.h"
 
 using namespace std;
@@ -134,8 +141,8 @@ public:
         return _vsize;
     }
 
-    unsigned long getResidentSize() {
-        return (unsigned long)_rss * 4 * 1024;
+    unsigned long getResidentSizeInPages() {
+        return (unsigned long)_rss;
     }
 
     int _pid;
@@ -269,7 +276,7 @@ public:
         while (fgets(fstr, 1023, f) != NULL && !feof(f)) {
             // until the end of the file
             fstr[strlen(fstr) < 1 ? 0 : strlen(fstr) - 1] = '\0';
-            if (strncmp(fstr, "processor\t:", 11) == 0)
+            if (strncmp(fstr, "processor ", 10) == 0 || strncmp(fstr, "processor\t:", 11) == 0)
                 ++procCount;
             if (strncmp(fstr, "cpu MHz\t\t:", 10) == 0)
                 freq = fstr + 11;
@@ -394,7 +401,7 @@ public:
             if (mongo::parseNumberFromString(meminfo, &systemMem).isOK()) {
                 return systemMem * 1024;  // convert from kB to bytes
             } else
-                log() << "Unable to collect system memory information" << endl;
+                log() << "Unable to collect system memory information";
         }
         return 0;
     }
@@ -409,6 +416,26 @@ bool ProcessInfo::supported() {
     return true;
 }
 
+// get the number of CPUs available to the current process
+boost::optional<unsigned long> ProcessInfo::getNumAvailableCores() {
+    cpu_set_t set;
+
+    if (sched_getaffinity(0, sizeof(cpu_set_t), &set) == 0) {
+#ifdef CPU_COUNT  // glibc >= 2.6 has CPU_COUNT defined
+        return CPU_COUNT(&set);
+#else
+        unsigned long count = 0;
+        for (size_t i = 0; i < CPU_SETSIZE; i++)
+            if (CPU_ISSET(i, &set))
+                count++;
+        if (count > 0)
+            return count;
+#endif
+    }
+
+    return boost::none;
+}
+
 int ProcessInfo::getVirtualMemorySize() {
     LinuxProc p(_pid);
     return (int)(p.getVirtualMemorySize() / (1024.0 * 1024));
@@ -416,7 +443,7 @@ int ProcessInfo::getVirtualMemorySize() {
 
 int ProcessInfo::getResidentSize() {
     LinuxProc p(_pid);
-    return (int)(p.getResidentSize() / (1024.0 * 1024));
+    return (int)((p.getResidentSizeInPages() * getPageSize()) / (1024.0 * 1024));
 }
 
 double ProcessInfo::getSystemMemoryPressurePercentage() {
@@ -424,13 +451,6 @@ double ProcessInfo::getSystemMemoryPressurePercentage() {
 }
 
 void ProcessInfo::getExtraInfo(BSONObjBuilder& info) {
-    // [dm] i don't think mallinfo works. (64 bit.)  ??
-    struct mallinfo malloc_info =
-        mallinfo();  // structure has same name as function that returns it. (see malloc.h)
-    info.append("heap_usage_bytes",
-                malloc_info.uordblks /*main arena*/ + malloc_info.hblkhd /*mmap blocks*/);
-    // docs claim hblkhd is included in uordblks but it isn't
-
     LinuxProc p(_pid);
     if (p._maj_flt <= std::numeric_limits<long long>::max())
         info.appendNumber("page_faults", static_cast<long long>(p._maj_flt));
@@ -452,7 +472,7 @@ void ProcessInfo::SystemInfo::collectSystemInfo() {
     LinuxSysHelper::getLinuxDistro(distroName, distroVersion);
 
     if (uname(&unameData) == -1) {
-        log() << "Unable to collect detailed system information: " << strerror(errno) << endl;
+        log() << "Unable to collect detailed system information: " << strerror(errno);
     }
 
     osType = "Linux";
@@ -525,7 +545,7 @@ bool ProcessInfo::blockCheckSupported() {
 bool ProcessInfo::blockInMemory(const void* start) {
     unsigned char x = 0;
     if (mincore(const_cast<void*>(alignToStartOfPage(start)), getPageSize(), &x)) {
-        log() << "mincore failed: " << errnoWithDescription() << endl;
+        log() << "mincore failed: " << errnoWithDescription();
         return 1;
     }
     return x & 0x1;
@@ -536,7 +556,7 @@ bool ProcessInfo::pagesInMemory(const void* start, size_t numPages, vector<char>
     if (mincore(const_cast<void*>(alignToStartOfPage(start)),
                 numPages * getPageSize(),
                 reinterpret_cast<unsigned char*>(&out->front()))) {
-        log() << "mincore failed: " << errnoWithDescription() << endl;
+        log() << "mincore failed: " << errnoWithDescription();
         return false;
     }
     for (size_t i = 0; i < numPages; ++i) {
